#!/usr/bin/env python

""" Bluegiga BGAPI/BGLib implementation

Changelog:
    2013-05-04 - Fixed single-item struct.unpack returns (@zwasson on Github)
    2013-04-28 - Fixed numerous uint8array/bd_addr command arg errors
               - Added 'debug' support
    2013-04-16 - Fixed 'bglib_on_idle' to be 'on_idle'
    2013-04-15 - Added wifi BGAPI support in addition to BLE BGAPI
               - Fixed references to 'this' instead of 'self'
    2013-04-11 - Initial release

============================================
Bluegiga BGLib Python interface library
2013-05-04 by Jeff Rowberg <jeff@rowberg.net>
Updates should (hopefully) always be available at https://github.com/jrowberg/bglib

============================================
BGLib Python interface library code is placed under the MIT license
Copyright (c) 2013 Jeff Rowberg

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
===============================================

"""

__author__ = "Jeff Rowberg"
__license__ = "MIT"
__version__ = "2013-05-04"
__email__ = "jeff@rowberg.net"

import struct


# thanks to Masaaki Shibata for Python event handler code
# http://www.emptypage.jp/notes/pyevent.en.html

class BGAPIEvent(object):

    def __init__(self, doc=None):
        self.__doc__ = doc

    def __get__(self, obj, objtype=None):
        if obj is None:
            return self
        return BGAPIEventHandler(self, obj)

    def __set__(self, obj, value):
        pass


class BGAPIEventHandler(object):

    def __init__(self, event, obj):

        self.event = event
        self.obj = obj

    def _getfunctionlist(self):

        """(internal use) """

        try:
            eventhandler = self.obj.__eventhandler__
        except AttributeError:
            eventhandler = self.obj.__eventhandler__ = {}
        return eventhandler.setdefault(self.event, [])

    def add(self, func):

        """Add new event handler function.

        Event handler function must be defined like func(sender, earg).
        You can add handler also by using '+=' operator.
        """

        self._getfunctionlist().append(func)
        return self

    def remove(self, func):

        """Remove existing event handler function.

        You can remove handler also by using '-=' operator.
        """

        self._getfunctionlist().remove(func)
        return self

    def fire(self, earg=None):

        """Fire event and call all handler functions

        You can call EventHandler object itself like e(earg) instead of
        e.fire(earg).
        """

        for func in self._getfunctionlist():
            func(self.obj, earg)

    __iadd__ = add
    __isub__ = remove
    __call__ = fire


class BGLib(object):

    def ble_cmd_system_reset(self, boot_in_dfu):
        return struct.pack('<4BB', 0, 1, 0, 0, boot_in_dfu)
    def ble_cmd_system_hello(self):
        return struct.pack('<4B', 0, 0, 0, 1)
    def ble_cmd_system_address_get(self):
        return struct.pack('<4B', 0, 0, 0, 2)
    def ble_cmd_system_reg_write(self, address, value):
        return struct.pack('<4BHB', 0, 3, 0, 3, address, value)
    def ble_cmd_system_reg_read(self, address):
        return struct.pack('<4BH', 0, 2, 0, 4, address)
    def ble_cmd_system_get_counters(self):
        return struct.pack('<4B', 0, 0, 0, 5)
    def ble_cmd_system_get_connections(self):
        return struct.pack('<4B', 0, 0, 0, 6)
    def ble_cmd_system_read_memory(self, address, length):
        return struct.pack('<4BIB', 0, 5, 0, 7, address, length)
    def ble_cmd_system_get_info(self):
        return struct.pack('<4B', 0, 0, 0, 8)
    def ble_cmd_system_endpoint_tx(self, endpoint, data):
        return struct.pack('<4BBB' + str(len(data)) + 's', 0, 2 + len(data), 0, 9, endpoint, len(data), bytes(i for i in data))
    def ble_cmd_system_whitelist_append(self, address, address_type):
        return struct.pack('<4B6sB', 0, 7, 0, 10, bytes(i for i in address), address_type)
    def ble_cmd_system_whitelist_remove(self, address, address_type):
        return struct.pack('<4B6sB', 0, 7, 0, 11, bytes(i for i in address), address_type)
    def ble_cmd_system_whitelist_clear(self):
        return struct.pack('<4B', 0, 0, 0, 12)
    def ble_cmd_system_endpoint_rx(self, endpoint, size):
        return struct.pack('<4BBB', 0, 2, 0, 13, endpoint, size)
    def ble_cmd_system_endpoint_set_watermarks(self, endpoint, rx, tx):
        return struct.pack('<4BBBB', 0, 3, 0, 14, endpoint, rx, tx)
    def ble_cmd_flash_ps_defrag(self):
        return struct.pack('<4B', 0, 0, 1, 0)
    def ble_cmd_flash_ps_dump(self):
        return struct.pack('<4B', 0, 0, 1, 1)
    def ble_cmd_flash_ps_erase_all(self):
        return struct.pack('<4B', 0, 0, 1, 2)
    def ble_cmd_flash_ps_save(self, key, value):
        return struct.pack('<4BHB' + str(len(value)) + 's', 0, 3 + len(value), 1, 3, key, len(value), bytes(i for i in value))
    def ble_cmd_flash_ps_load(self, key):
        return struct.pack('<4BH', 0, 2, 1, 4, key)
    def ble_cmd_flash_ps_erase(self, key):
        return struct.pack('<4BH', 0, 2, 1, 5, key)
    def ble_cmd_flash_erase_page(self, page):
        return struct.pack('<4BB', 0, 1, 1, 6, page)
    def ble_cmd_flash_write_words(self, address, words):
        return struct.pack('<4BHB' + str(len(words)) + 's', 0, 3 + len(words), 1, 7, address, len(words), bytes(i for i in words))
    def ble_cmd_attributes_write(self, handle, offset, value):
        return struct.pack('<4BHBB' + str(len(value)) + 's', 0, 4 + len(value), 2, 0, handle, offset, len(value), bytes(i for i in value))
    def ble_cmd_attributes_read(self, handle, offset):
        return struct.pack('<4BHH', 0, 4, 2, 1, handle, offset)
    def ble_cmd_attributes_read_type(self, handle):
        return struct.pack('<4BH', 0, 2, 2, 2, handle)
    def ble_cmd_attributes_user_read_response(self, connection, att_error, value):
        return struct.pack('<4BBBB' + str(len(value)) + 's', 0, 3 + len(value), 2, 3, connection, att_error, len(value), bytes(i for i in value))
    def ble_cmd_attributes_user_write_response(self, connection, att_error):
        return struct.pack('<4BBB', 0, 2, 2, 4, connection, att_error)
    def ble_cmd_connection_disconnect(self, connection):
        return struct.pack('<4BB', 0, 1, 3, 0, connection)
    def ble_cmd_connection_get_rssi(self, connection):
        return struct.pack('<4BB', 0, 1, 3, 1, connection)
    def ble_cmd_connection_update(self, connection, interval_min, interval_max, latency, timeout):
        return struct.pack('<4BBHHHH', 0, 9, 3, 2, connection, interval_min, interval_max, latency, timeout)
    def ble_cmd_connection_version_update(self, connection):
        return struct.pack('<4BB', 0, 1, 3, 3, connection)
    def ble_cmd_connection_channel_map_get(self, connection):
        return struct.pack('<4BB', 0, 1, 3, 4, connection)
    def ble_cmd_connection_channel_map_set(self, connection, map):
        return struct.pack('<4BBB' + str(len(map)) + 's', 0, 2 + len(map), 3, 5, connection, len(map), bytes(i for i in map))
    def ble_cmd_connection_features_get(self, connection):
        return struct.pack('<4BB', 0, 1, 3, 6, connection)
    def ble_cmd_connection_get_status(self, connection):
        return struct.pack('<4BB', 0, 1, 3, 7, connection)
    def ble_cmd_connection_raw_tx(self, connection, data):
        return struct.pack('<4BBB' + str(len(data)) + 's', 0, 2 + len(data), 3, 8, connection, len(data), bytes(i for i in data))
    def ble_cmd_attclient_find_by_type_value(self, connection, start, end, uuid, value):
        return struct.pack('<4BBHHHB' + str(len(value)) + 's', 0, 8 + len(value), 4, 0, connection, start, end, uuid, len(value), bytes(i for i in value))
    def ble_cmd_attclient_read_by_group_type(self, connection, start, end, uuid):
        return struct.pack('<4BBHHB' + str(len(uuid)) + 's', 0, 6 + len(uuid), 4, 1, connection, start, end, len(uuid), bytes(i for i in uuid))
    def ble_cmd_attclient_read_by_type(self, connection, start, end, uuid):
        return struct.pack('<4BBHHB' + str(len(uuid)) + 's', 0, 6 + len(uuid), 4, 2, connection, start, end, len(uuid), bytes(i for i in uuid))
    def ble_cmd_attclient_find_information(self, connection, start, end):
        return struct.pack('<4BBHH', 0, 5, 4, 3, connection, start, end)
    def ble_cmd_attclient_read_by_handle(self, connection, chrhandle):
        return struct.pack('<4BBH', 0, 3, 4, 4, connection, chrhandle)
    def ble_cmd_attclient_attribute_write(self, connection, atthandle, data):
        return struct.pack('<4BBHB' + str(len(data)) + 's', 0, 4 + len(data), 4, 5, connection, atthandle, len(data), bytes(i for i in data))
    def ble_cmd_attclient_write_command(self, connection, atthandle, data):
        return struct.pack('<4BBHB' + str(len(data)) + 's', 0, 4 + len(data), 4, 6, connection, atthandle, len(data), bytes(i for i in data))
    def ble_cmd_attclient_indicate_confirm(self, connection):
        return struct.pack('<4BB', 0, 1, 4, 7, connection)
    def ble_cmd_attclient_read_long(self, connection, chrhandle):
        return struct.pack('<4BBH', 0, 3, 4, 8, connection, chrhandle)
    def ble_cmd_attclient_prepare_write(self, connection, atthandle, offset, data):
        return struct.pack('<4BBHHB' + str(len(data)) + 's', 0, 6 + len(data), 4, 9, connection, atthandle, offset, len(data), bytes(i for i in data))
    def ble_cmd_attclient_execute_write(self, connection, commit):
        return struct.pack('<4BBB', 0, 2, 4, 10, connection, commit)
    def ble_cmd_attclient_read_multiple(self, connection, handles):
        return struct.pack('<4BBB' + str(len(handles)) + 's', 0, 2 + len(handles), 4, 11, connection, len(handles), bytes(i for i in handles))
    def ble_cmd_sm_encrypt_start(self, handle, bonding):
        return struct.pack('<4BBB', 0, 2, 5, 0, handle, bonding)
    def ble_cmd_sm_set_bondable_mode(self, bondable):
        return struct.pack('<4BB', 0, 1, 5, 1, bondable)
    def ble_cmd_sm_delete_bonding(self, handle):
        return struct.pack('<4BB', 0, 1, 5, 2, handle)
    def ble_cmd_sm_set_parameters(self, mitm, min_key_size, io_capabilities):
        return struct.pack('<4BBBB', 0, 3, 5, 3, mitm, min_key_size, io_capabilities)
    def ble_cmd_sm_passkey_entry(self, handle, passkey):
        return struct.pack('<4BBI', 0, 5, 5, 4, handle, passkey)
    def ble_cmd_sm_get_bonds(self):
        return struct.pack('<4B', 0, 0, 5, 5)
    def ble_cmd_sm_set_oob_data(self, oob):
        return struct.pack('<4BB' + str(len(oob)) + 's', 0, 1 + len(oob), 5, 6, len(oob), bytes(i for i in oob))
    def ble_cmd_gap_set_privacy_flags(self, peripheral_privacy, central_privacy):
        return struct.pack('<4BBB', 0, 2, 6, 0, peripheral_privacy, central_privacy)
    def ble_cmd_gap_set_mode(self, discover, connect):
        return struct.pack('<4BBB', 0, 2, 6, 1, discover, connect)
    def ble_cmd_gap_discover(self, mode):
        return struct.pack('<4BB', 0, 1, 6, 2, mode)
    def ble_cmd_gap_connect_direct(self, address, addr_type, conn_interval_min, conn_interval_max, timeout, latency):
        return struct.pack('<4B6sBHHHH', 0, 15, 6, 3, bytes(i for i in address), addr_type, conn_interval_min, conn_interval_max, timeout, latency)
    def ble_cmd_gap_end_procedure(self):
        return struct.pack('<4B', 0, 0, 6, 4)
    def ble_cmd_gap_connect_selective(self, conn_interval_min, conn_interval_max, timeout, latency):
        return struct.pack('<4BHHHH', 0, 8, 6, 5, conn_interval_min, conn_interval_max, timeout, latency)
    def ble_cmd_gap_set_filtering(self, scan_policy, adv_policy, scan_duplicate_filtering):
        return struct.pack('<4BBBB', 0, 3, 6, 6, scan_policy, adv_policy, scan_duplicate_filtering)
    def ble_cmd_gap_set_scan_parameters(self, scan_interval, scan_window, active):
        return struct.pack('<4BHHB', 0, 5, 6, 7, scan_interval, scan_window, active)
    def ble_cmd_gap_set_adv_parameters(self, adv_interval_min, adv_interval_max, adv_channels):
        return struct.pack('<4BHHB', 0, 5, 6, 8, adv_interval_min, adv_interval_max, adv_channels)
    def ble_cmd_gap_set_adv_data(self, set_scanrsp, adv_data):
        return struct.pack('<4BBB' + str(len(adv_data)) + 's', 0, 2 + len(adv_data), 6, 9, set_scanrsp, len(adv_data), bytes(i for i in adv_data))
    def ble_cmd_gap_set_directed_connectable_mode(self, address, addr_type):
        return struct.pack('<4B6sB', 0, 7, 6, 10, bytes(i for i in address), addr_type)
    def ble_cmd_hardware_io_port_config_irq(self, port, enable_bits, falling_edge):
        return struct.pack('<4BBBB', 0, 3, 7, 0, port, enable_bits, falling_edge)
    def ble_cmd_hardware_set_soft_timer(self, time, handle, single_shot):
        return struct.pack('<4BIBB', 0, 6, 7, 1, time, handle, single_shot)
    def ble_cmd_hardware_adc_read(self, input, decimation, reference_selection):
        return struct.pack('<4BBBB', 0, 3, 7, 2, input, decimation, reference_selection)
    def ble_cmd_hardware_io_port_config_direction(self, port, direction):
        return struct.pack('<4BBB', 0, 2, 7, 3, port, direction)
    def ble_cmd_hardware_io_port_config_function(self, port, function):
        return struct.pack('<4BBB', 0, 2, 7, 4, port, function)
    def ble_cmd_hardware_io_port_config_pull(self, port, tristate_mask, pull_up):
        return struct.pack('<4BBBB', 0, 3, 7, 5, port, tristate_mask, pull_up)
    def ble_cmd_hardware_io_port_write(self, port, mask, data):
        return struct.pack('<4BBBB', 0, 3, 7, 6, port, mask, data)
    def ble_cmd_hardware_io_port_read(self, port, mask):
        return struct.pack('<4BBB', 0, 2, 7, 7, port, mask)
    def ble_cmd_hardware_spi_config(self, channel, polarity, phase, bit_order, baud_e, baud_m):
        return struct.pack('<4BBBBBBB', 0, 6, 7, 8, channel, polarity, phase, bit_order, baud_e, baud_m)
    def ble_cmd_hardware_spi_transfer(self, channel, data):
        return struct.pack('<4BBB' + str(len(data)) + 's', 0, 2 + len(data), 7, 9, channel, len(data), bytes(i for i in data))
    def ble_cmd_hardware_i2c_read(self, address, stop, length):
        return struct.pack('<4BBBB', 0, 3, 7, 10, address, stop, length)
    def ble_cmd_hardware_i2c_write(self, address, stop, data):
        return struct.pack('<4BBBB' + str(len(data)) + 's', 0, 3 + len(data), 7, 11, address, stop, len(data), bytes(i for i in data))
    def ble_cmd_hardware_set_txpower(self, power):
        return struct.pack('<4BB', 0, 1, 7, 12, power)
    def ble_cmd_hardware_timer_comparator(self, timer, channel, mode, comparator_value):
        return struct.pack('<4BBBBH', 0, 5, 7, 13, timer, channel, mode, comparator_value)
    def ble_cmd_test_phy_tx(self, channel, length, type):
        return struct.pack('<4BBBB', 0, 3, 8, 0, channel, length, type)
    def ble_cmd_test_phy_rx(self, channel):
        return struct.pack('<4BB', 0, 1, 8, 1, channel)
    def ble_cmd_test_phy_end(self):
        return struct.pack('<4B', 0, 0, 8, 2)
    def ble_cmd_test_phy_reset(self):
        return struct.pack('<4B', 0, 0, 8, 3)
    def ble_cmd_test_get_channel_map(self):
        return struct.pack('<4B', 0, 0, 8, 4)
    def ble_cmd_test_debug(self, input):
        return struct.pack('<4BB' + str(len(input)) + 's', 0, 1 + len(input), 8, 5, len(input), bytes(i for i in input))

    ble_rsp_system_reset = BGAPIEvent()
    ble_rsp_system_hello = BGAPIEvent()
    ble_rsp_system_address_get = BGAPIEvent()
    ble_rsp_system_reg_write = BGAPIEvent()
    ble_rsp_system_reg_read = BGAPIEvent()
    ble_rsp_system_get_counters = BGAPIEvent()
    ble_rsp_system_get_connections = BGAPIEvent()
    ble_rsp_system_read_memory = BGAPIEvent()
    ble_rsp_system_get_info = BGAPIEvent()
    ble_rsp_system_endpoint_tx = BGAPIEvent()
    ble_rsp_system_whitelist_append = BGAPIEvent()
    ble_rsp_system_whitelist_remove = BGAPIEvent()
    ble_rsp_system_whitelist_clear = BGAPIEvent()
    ble_rsp_system_endpoint_rx = BGAPIEvent()
    ble_rsp_system_endpoint_set_watermarks = BGAPIEvent()
    ble_rsp_flash_ps_defrag = BGAPIEvent()
    ble_rsp_flash_ps_dump = BGAPIEvent()
    ble_rsp_flash_ps_erase_all = BGAPIEvent()
    ble_rsp_flash_ps_save = BGAPIEvent()
    ble_rsp_flash_ps_load = BGAPIEvent()
    ble_rsp_flash_ps_erase = BGAPIEvent()
    ble_rsp_flash_erase_page = BGAPIEvent()
    ble_rsp_flash_write_words = BGAPIEvent()
    ble_rsp_attributes_write = BGAPIEvent()
    ble_rsp_attributes_read = BGAPIEvent()
    ble_rsp_attributes_read_type = BGAPIEvent()
    ble_rsp_attributes_user_read_response = BGAPIEvent()
    ble_rsp_attributes_user_write_response = BGAPIEvent()
    ble_rsp_connection_disconnect = BGAPIEvent()
    ble_rsp_connection_get_rssi = BGAPIEvent()
    ble_rsp_connection_update = BGAPIEvent()
    ble_rsp_connection_version_update = BGAPIEvent()
    ble_rsp_connection_channel_map_get = BGAPIEvent()
    ble_rsp_connection_channel_map_set = BGAPIEvent()
    ble_rsp_connection_features_get = BGAPIEvent()
    ble_rsp_connection_get_status = BGAPIEvent()
    ble_rsp_connection_raw_tx = BGAPIEvent()
    ble_rsp_attclient_find_by_type_value = BGAPIEvent()
    ble_rsp_attclient_read_by_group_type = BGAPIEvent()
    ble_rsp_attclient_read_by_type = BGAPIEvent()
    ble_rsp_attclient_find_information = BGAPIEvent()
    ble_rsp_attclient_read_by_handle = BGAPIEvent()
    ble_rsp_attclient_attribute_write = BGAPIEvent()
    ble_rsp_attclient_write_command = BGAPIEvent()
    ble_rsp_attclient_indicate_confirm = BGAPIEvent()
    ble_rsp_attclient_read_long = BGAPIEvent()
    ble_rsp_attclient_prepare_write = BGAPIEvent()
    ble_rsp_attclient_execute_write = BGAPIEvent()
    ble_rsp_attclient_read_multiple = BGAPIEvent()
    ble_rsp_sm_encrypt_start = BGAPIEvent()
    ble_rsp_sm_set_bondable_mode = BGAPIEvent()
    ble_rsp_sm_delete_bonding = BGAPIEvent()
    ble_rsp_sm_set_parameters = BGAPIEvent()
    ble_rsp_sm_passkey_entry = BGAPIEvent()
    ble_rsp_sm_get_bonds = BGAPIEvent()
    ble_rsp_sm_set_oob_data = BGAPIEvent()
    ble_rsp_gap_set_privacy_flags = BGAPIEvent()
    ble_rsp_gap_set_mode = BGAPIEvent()
    ble_rsp_gap_discover = BGAPIEvent()
    ble_rsp_gap_connect_direct = BGAPIEvent()
    ble_rsp_gap_end_procedure = BGAPIEvent()
    ble_rsp_gap_connect_selective = BGAPIEvent()
    ble_rsp_gap_set_filtering = BGAPIEvent()
    ble_rsp_gap_set_scan_parameters = BGAPIEvent()
    ble_rsp_gap_set_adv_parameters = BGAPIEvent()
    ble_rsp_gap_set_adv_data = BGAPIEvent()
    ble_rsp_gap_set_directed_connectable_mode = BGAPIEvent()
    ble_rsp_hardware_io_port_config_irq = BGAPIEvent()
    ble_rsp_hardware_set_soft_timer = BGAPIEvent()
    ble_rsp_hardware_adc_read = BGAPIEvent()
    ble_rsp_hardware_io_port_config_direction = BGAPIEvent()
    ble_rsp_hardware_io_port_config_function = BGAPIEvent()
    ble_rsp_hardware_io_port_config_pull = BGAPIEvent()
    ble_rsp_hardware_io_port_write = BGAPIEvent()
    ble_rsp_hardware_io_port_read = BGAPIEvent()
    ble_rsp_hardware_spi_config = BGAPIEvent()
    ble_rsp_hardware_spi_transfer = BGAPIEvent()
    ble_rsp_hardware_i2c_read = BGAPIEvent()
    ble_rsp_hardware_i2c_write = BGAPIEvent()
    ble_rsp_hardware_set_txpower = BGAPIEvent()
    ble_rsp_hardware_timer_comparator = BGAPIEvent()
    ble_rsp_test_phy_tx = BGAPIEvent()
    ble_rsp_test_phy_rx = BGAPIEvent()
    ble_rsp_test_phy_end = BGAPIEvent()
    ble_rsp_test_phy_reset = BGAPIEvent()
    ble_rsp_test_get_channel_map = BGAPIEvent()
    ble_rsp_test_debug = BGAPIEvent()

    ble_evt_system_boot = BGAPIEvent()
    ble_evt_system_debug = BGAPIEvent()
    ble_evt_system_endpoint_watermark_rx = BGAPIEvent()
    ble_evt_system_endpoint_watermark_tx = BGAPIEvent()
    ble_evt_system_script_failure = BGAPIEvent()
    ble_evt_system_no_license_key = BGAPIEvent()
    ble_evt_flash_ps_key = BGAPIEvent()
    ble_evt_attributes_value = BGAPIEvent()
    ble_evt_attributes_user_read_request = BGAPIEvent()
    ble_evt_attributes_status = BGAPIEvent()
    ble_evt_connection_status = BGAPIEvent()
    ble_evt_connection_version_ind = BGAPIEvent()
    ble_evt_connection_feature_ind = BGAPIEvent()
    ble_evt_connection_raw_rx = BGAPIEvent()
    ble_evt_connection_disconnected = BGAPIEvent()
    ble_evt_attclient_indicated = BGAPIEvent()
    ble_evt_attclient_procedure_completed = BGAPIEvent()
    ble_evt_attclient_group_found = BGAPIEvent()
    ble_evt_attclient_attribute_found = BGAPIEvent()
    ble_evt_attclient_find_information_found = BGAPIEvent()
    ble_evt_attclient_attribute_value = BGAPIEvent()
    ble_evt_attclient_read_multiple_response = BGAPIEvent()
    ble_evt_sm_smp_data = BGAPIEvent()
    ble_evt_sm_bonding_fail = BGAPIEvent()
    ble_evt_sm_passkey_display = BGAPIEvent()
    ble_evt_sm_passkey_request = BGAPIEvent()
    ble_evt_sm_bond_status = BGAPIEvent()
    ble_evt_gap_scan_response = BGAPIEvent()
    ble_evt_gap_mode_changed = BGAPIEvent()
    ble_evt_hardware_io_port_status = BGAPIEvent()
    ble_evt_hardware_soft_timer = BGAPIEvent()
    ble_evt_hardware_adc_result = BGAPIEvent()

    def wifi_cmd_dfu_reset(self, dfu):
        return struct.pack('<4BB', 0, 1, 0, 0, dfu)
    def wifi_cmd_dfu_flash_set_address(self, address):
        return struct.pack('<4BI', 0, 4, 0, 1, address)
    def wifi_cmd_dfu_flash_upload(self):
        return struct.pack('<4BB' + str(len(data)) + 's', 0, 1 + len(data), 0, 2, data, len(data), bytes(i for i in data))
    def wifi_cmd_dfu_flash_upload_finish(self):
        return struct.pack('<4B', 0, 0, 0, 3)
    def wifi_cmd_system_sync(self):
        return struct.pack('<4B', 0, 0, 1, 0)
    def wifi_cmd_system_reset(self, dfu):
        return struct.pack('<4BB', 0, 1, 1, 1, dfu)
    def wifi_cmd_system_hello(self):
        return struct.pack('<4B', 0, 0, 1, 2)
    def wifi_cmd_system_set_max_power_saving_state(self, state):
        return struct.pack('<4BB', 0, 1, 1, 3, state)
    def wifi_cmd_config_get_mac(self, hw_interface):
        return struct.pack('<4BB', 0, 1, 2, 0, hw_interface)
    def wifi_cmd_config_set_mac(self, hw_interface):
        return struct.pack('<4BB', 0, 1, 2, 1, hw_interface, mac)
    def wifi_cmd_sme_wifi_on(self):
        return struct.pack('<4B', 0, 0, 3, 0)
    def wifi_cmd_sme_wifi_off(self):
        return struct.pack('<4B', 0, 0, 3, 1)
    def wifi_cmd_sme_power_on(self, enable):
        return struct.pack('<4BB', 0, 1, 3, 2, enable)
    def wifi_cmd_sme_start_scan(self, hw_interface):
        return struct.pack('<4BBB' + str(len(chList)) + 's', 0, 2 + len(chList), 3, 3, hw_interface, chList, len(chList), bytes(i for i in chList))
    def wifi_cmd_sme_stop_scan(self):
        return struct.pack('<4B', 0, 0, 3, 4)
    def wifi_cmd_sme_set_password(self):
        return struct.pack('<4BB' + str(len(password)) + 's', 0, 1 + len(password), 3, 5, password, len(password), bytes(i for i in password))
    def wifi_cmd_sme_connect_bssid(self):
        return struct.pack('<4B', 0, 0, 3, 6, bssid)
    def wifi_cmd_sme_connect_ssid(self):
        return struct.pack('<4BB' + str(len(ssid)) + 's', 0, 1 + len(ssid), 3, 7, ssid, len(ssid), bytes(i for i in ssid))
    def wifi_cmd_sme_disconnect(self):
        return struct.pack('<4B', 0, 0, 3, 8)
    def wifi_cmd_sme_set_scan_channels(self, hw_interface):
        return struct.pack('<4BBB' + str(len(chList)) + 's', 0, 2 + len(chList), 3, 9, hw_interface, chList, len(chList), bytes(i for i in chList))
    def wifi_cmd_tcpip_start_tcp_server(self, port, default_destination):
        return struct.pack('<4BHb', 0, 3, 4, 0, port, default_destination)
    def wifi_cmd_tcpip_tcp_connect(self, port, routing):
        return struct.pack('<4BHb', 0, 3, 4, 1, address, port, routing)
    def wifi_cmd_tcpip_start_udp_server(self, port, default_destination):
        return struct.pack('<4BHb', 0, 3, 4, 2, port, default_destination)
    def wifi_cmd_tcpip_udp_connect(self, port, routing):
        return struct.pack('<4BHb', 0, 3, 4, 3, address, port, routing)
    def wifi_cmd_tcpip_configure(self, use_dhcp):
        return struct.pack('<4BB', 0, 1, 4, 4, address, netmask, gateway, use_dhcp)
    def wifi_cmd_tcpip_dns_configure(self, index):
        return struct.pack('<4BB', 0, 1, 4, 5, index, address)
    def wifi_cmd_tcpip_dns_gethostbyname(self):
        return struct.pack('<4BB' + str(len(name)) + 's', 0, 1 + len(name), 4, 6, name, len(name), bytes(i for i in name))
    def wifi_cmd_endpoint_send(self, endpoint):
        return struct.pack('<4BBB' + str(len(data)) + 's', 0, 2 + len(data), 5, 0, endpoint, data, len(data), bytes(i for i in data))
    def wifi_cmd_endpoint_set_streaming(self, endpoint, streaming):
        return struct.pack('<4BBB', 0, 2, 5, 1, endpoint, streaming)
    def wifi_cmd_endpoint_set_active(self, endpoint, active):
        return struct.pack('<4BBB', 0, 2, 5, 2, endpoint, active)
    def wifi_cmd_endpoint_set_streaming_destination(self, endpoint, streaming_destination):
        return struct.pack('<4BBb', 0, 2, 5, 3, endpoint, streaming_destination)
    def wifi_cmd_endpoint_close(self, endpoint):
        return struct.pack('<4BB', 0, 1, 5, 4, endpoint)
    def wifi_cmd_hardware_set_soft_timer(self, time, handle, single_shot):
        return struct.pack('<4BIBB', 0, 6, 6, 0, time, handle, single_shot)
    def wifi_cmd_hardware_external_interrupt_config(self, enable, polarity):
        return struct.pack('<4BBB', 0, 2, 6, 1, enable, polarity)
    def wifi_cmd_hardware_change_notification_config(self, enable):
        return struct.pack('<4BI', 0, 4, 6, 2, enable)
    def wifi_cmd_hardware_change_notification_pullup(self, pullup):
        return struct.pack('<4BI', 0, 4, 6, 3, pullup)
    def wifi_cmd_hardware_io_port_config_direction(self, port, mask, direction):
        return struct.pack('<4BBHH', 0, 5, 6, 4, port, mask, direction)
    def wifi_cmd_hardware_io_port_config_open_drain(self, port, mask, open_drain):
        return struct.pack('<4BBHH', 0, 5, 6, 5, port, mask, open_drain)
    def wifi_cmd_hardware_io_port_write(self, port, mask, data):
        return struct.pack('<4BBHH', 0, 5, 6, 6, port, mask, data)
    def wifi_cmd_hardware_io_port_read(self, port, mask):
        return struct.pack('<4BBH', 0, 3, 6, 7, port, mask)
    def wifi_cmd_hardware_output_compare(self, index, bit32, timer, mode, compare_value):
        return struct.pack('<4BBBBBI', 0, 8, 6, 8, index, bit32, timer, mode, compare_value)
    def wifi_cmd_hardware_adc_read(self, input):
        return struct.pack('<4BB', 0, 1, 6, 9, input)
    def wifi_cmd_flash_ps_defrag(self):
        return struct.pack('<4B', 0, 0, 7, 0)
    def wifi_cmd_flash_ps_dump(self):
        return struct.pack('<4B', 0, 0, 7, 1)
    def wifi_cmd_flash_ps_erase_all(self):
        return struct.pack('<4B', 0, 0, 7, 2)
    def wifi_cmd_flash_ps_save(self, key):
        return struct.pack('<4BHB' + str(len(value)) + 's', 0, 3 + len(value), 7, 3, key, value, len(value), bytes(i for i in value))
    def wifi_cmd_flash_ps_load(self, key):
        return struct.pack('<4BH', 0, 2, 7, 4, key)
    def wifi_cmd_flash_ps_erase(self, key):
        return struct.pack('<4BH', 0, 2, 7, 5, key)
    def wifi_cmd_i2c_start_read(self, endpoint, slave_address, length):
        return struct.pack('<4BBHB', 0, 4, 8, 0, endpoint, slave_address, length)
    def wifi_cmd_i2c_start_write(self, endpoint, slave_address):
        return struct.pack('<4BBH', 0, 3, 8, 1, endpoint, slave_address)
    def wifi_cmd_i2c_stop(self, endpoint):
        return struct.pack('<4BB', 0, 1, 8, 2, endpoint)

    wifi_rsp_dfu_reset = BGAPIEvent()
    wifi_rsp_dfu_flash_set_address = BGAPIEvent()
    wifi_rsp_dfu_flash_upload = BGAPIEvent()
    wifi_rsp_dfu_flash_upload_finish = BGAPIEvent()
    wifi_rsp_system_sync = BGAPIEvent()
    wifi_rsp_system_reset = BGAPIEvent()
    wifi_rsp_system_hello = BGAPIEvent()
    wifi_rsp_system_set_max_power_saving_state = BGAPIEvent()
    wifi_rsp_config_get_mac = BGAPIEvent()
    wifi_rsp_config_set_mac = BGAPIEvent()
    wifi_rsp_sme_wifi_on = BGAPIEvent()
    wifi_rsp_sme_wifi_off = BGAPIEvent()
    wifi_rsp_sme_power_on = BGAPIEvent()
    wifi_rsp_sme_start_scan = BGAPIEvent()
    wifi_rsp_sme_stop_scan = BGAPIEvent()
    wifi_rsp_sme_set_password = BGAPIEvent()
    wifi_rsp_sme_connect_bssid = BGAPIEvent()
    wifi_rsp_sme_connect_ssid = BGAPIEvent()
    wifi_rsp_sme_disconnect = BGAPIEvent()
    wifi_rsp_sme_set_scan_channels = BGAPIEvent()
    wifi_rsp_tcpip_start_tcp_server = BGAPIEvent()
    wifi_rsp_tcpip_tcp_connect = BGAPIEvent()
    wifi_rsp_tcpip_start_udp_server = BGAPIEvent()
    wifi_rsp_tcpip_udp_connect = BGAPIEvent()
    wifi_rsp_tcpip_configure = BGAPIEvent()
    wifi_rsp_tcpip_dns_configure = BGAPIEvent()
    wifi_rsp_tcpip_dns_gethostbyname = BGAPIEvent()
    wifi_rsp_endpoint_send = BGAPIEvent()
    wifi_rsp_endpoint_set_streaming = BGAPIEvent()
    wifi_rsp_endpoint_set_active = BGAPIEvent()
    wifi_rsp_endpoint_set_streaming_destination = BGAPIEvent()
    wifi_rsp_endpoint_close = BGAPIEvent()
    wifi_rsp_hardware_set_soft_timer = BGAPIEvent()
    wifi_rsp_hardware_external_interrupt_config = BGAPIEvent()
    wifi_rsp_hardware_change_notification_config = BGAPIEvent()
    wifi_rsp_hardware_change_notification_pullup = BGAPIEvent()
    wifi_rsp_hardware_io_port_config_direction = BGAPIEvent()
    wifi_rsp_hardware_io_port_config_open_drain = BGAPIEvent()
    wifi_rsp_hardware_io_port_write = BGAPIEvent()
    wifi_rsp_hardware_io_port_read = BGAPIEvent()
    wifi_rsp_hardware_output_compare = BGAPIEvent()
    wifi_rsp_hardware_adc_read = BGAPIEvent()
    wifi_rsp_flash_ps_defrag = BGAPIEvent()
    wifi_rsp_flash_ps_dump = BGAPIEvent()
    wifi_rsp_flash_ps_erase_all = BGAPIEvent()
    wifi_rsp_flash_ps_save = BGAPIEvent()
    wifi_rsp_flash_ps_load = BGAPIEvent()
    wifi_rsp_flash_ps_erase = BGAPIEvent()
    wifi_rsp_i2c_start_read = BGAPIEvent()
    wifi_rsp_i2c_start_write = BGAPIEvent()
    wifi_rsp_i2c_stop = BGAPIEvent()

    wifi_evt_dfu_boot = BGAPIEvent()
    wifi_evt_system_boot = BGAPIEvent()
    wifi_evt_system_state = BGAPIEvent()
    wifi_evt_system_sw_exception = BGAPIEvent()
    wifi_evt_system_power_saving_state = BGAPIEvent()
    wifi_evt_config_mac_address = BGAPIEvent()
    wifi_evt_sme_wifi_is_on = BGAPIEvent()
    wifi_evt_sme_wifi_is_off = BGAPIEvent()
    wifi_evt_sme_scan_result = BGAPIEvent()
    wifi_evt_sme_scan_result_drop = BGAPIEvent()
    wifi_evt_sme_scanned = BGAPIEvent()
    wifi_evt_sme_connected = BGAPIEvent()
    wifi_evt_sme_disconnected = BGAPIEvent()
    wifi_evt_sme_interface_status = BGAPIEvent()
    wifi_evt_sme_connect_failed = BGAPIEvent()
    wifi_evt_sme_connect_retry = BGAPIEvent()
    wifi_evt_tcpip_configuration = BGAPIEvent()
    wifi_evt_tcpip_dns_configuration = BGAPIEvent()
    wifi_evt_tcpip_endpoint_status = BGAPIEvent()
    wifi_evt_tcpip_dns_gethostbyname_result = BGAPIEvent()
    wifi_evt_endpoint_syntax_error = BGAPIEvent()
    wifi_evt_endpoint_data = BGAPIEvent()
    wifi_evt_endpoint_status = BGAPIEvent()
    wifi_evt_endpoint_closing = BGAPIEvent()
    wifi_evt_hardware_soft_timer = BGAPIEvent()
    wifi_evt_hardware_change_notification = BGAPIEvent()
    wifi_evt_hardware_external_interrupt = BGAPIEvent()
    wifi_evt_flash_ps_key = BGAPIEvent()

    on_busy = BGAPIEvent()
    on_idle = BGAPIEvent()
    on_timeout = BGAPIEvent()
    on_before_tx_command = BGAPIEvent()
    on_tx_command_complete = BGAPIEvent()

    bgapi_rx_buffer = b""
    bgapi_rx_expected_length = 0
    busy = False
    packet_mode = False
    debug = False

    def send_command(self, ser, packet):
        if self.packet_mode: packet = chr(len(packet) & 0xFF) + packet
        if self.debug: print('=>[ ' + ' '.join(['%02X' % b for b in packet]) + ' ]')
        self.on_before_tx_command()
        self.busy = True
        self.on_busy()
        ser.write(packet)
        self.on_tx_command_complete()

    def check_activity(self, ser, timeout=0):
        if timeout > 0:
            ser.timeout = timeout
            while 1:
                x = ser.read()
                if len(x) > 0:
                    self.parse(x)
                else: # timeout
                    self.busy = False
                    self.on_idle()
                    self.on_timeout()
                if not self.busy: # finished
                    break
        else:
            while ser.inWaiting(): self.parse(ser.read())
        return self.busy

    def parse(self, barray):
        b=barray[0]
        if len(self.bgapi_rx_buffer) == 0 and (b == 0x00 or b == 0x80 or b == 0x08 or b == 0x88):
            self.bgapi_rx_buffer+=bytes([b])
        elif len(self.bgapi_rx_buffer) == 1:
            self.bgapi_rx_buffer+=bytes([b])
            self.bgapi_rx_expected_length = 4 + (self.bgapi_rx_buffer[0] & 0x07) + self.bgapi_rx_buffer[1]
        elif len(self.bgapi_rx_buffer) > 1:
            self.bgapi_rx_buffer+=bytes([b])

        """
        BGAPI packet structure (as of 2012-11-07):
            Byte 0:
                  [7] - 1 bit, Message Type (MT)         0 = Command/Response, 1 = Event
                [6:3] - 4 bits, Technology Type (TT)     0000 = Bluetooth 4.0 single mode, 0001 = Wi-Fi
                [2:0] - 3 bits, Length High (LH)         Payload length (high bits)
            Byte 1:     8 bits, Length Low (LL)          Payload length (low bits)
            Byte 2:     8 bits, Class ID (CID)           Command class ID
            Byte 3:     8 bits, Command ID (CMD)         Command ID
            Bytes 4-n:  0 - 2048 Bytes, Payload (PL)     Up to 2048 bytes of payload
        """

        #print'%02X: %d, %d' % (b, len(self.bgapi_rx_buffer), self.bgapi_rx_expected_length)
        if self.bgapi_rx_expected_length > 0 and len(self.bgapi_rx_buffer) == self.bgapi_rx_expected_length:
            if self.debug: print('<=[ ' + ' '.join(['%02X' % b for b in self.bgapi_rx_buffer ]) + ' ]')
            packet_type, payload_length, packet_class, packet_command = self.bgapi_rx_buffer[:4]
            self.bgapi_rx_payload = self.bgapi_rx_buffer[4:]
            self.bgapi_rx_buffer = b""
            if packet_type & 0x88 == 0x00:
                # 0x00 = BLE response packet
                if packet_class == 0:
                    if packet_command == 0: # ble_rsp_system_reset
                        self.ble_rsp_system_reset({  })
                        self.busy = False
                        self.on_idle()
                    elif packet_command == 1: # ble_rsp_system_hello
                        self.ble_rsp_system_hello({  })
                    elif packet_command == 2: # ble_rsp_system_address_get
                        address = struct.unpack('<6s', self.bgapi_rx_payload[:6])[0]
                        address = address
                        self.ble_rsp_system_address_get({ 'address': address })
                    elif packet_command == 3: # ble_rsp_system_reg_write
                        result = struct.unpack('<H', self.bgapi_rx_payload[:2])[0]
                        self.ble_rsp_system_reg_write({ 'result': result })
                    elif packet_command == 4: # ble_rsp_system_reg_read
                        address, value = struct.unpack('<HB', self.bgapi_rx_payload[:3])
                        self.ble_rsp_system_reg_read({ 'address': address, 'value': value })
                    elif packet_command == 5: # ble_rsp_system_get_counters
                        txok, txretry, rxok, rxfail, mbuf = struct.unpack('<BBBBB', self.bgapi_rx_payload[:5])
                        self.ble_rsp_system_get_counters({ 'txok': txok, 'txretry': txretry, 'rxok': rxok, 'rxfail': rxfail, 'mbuf': mbuf })
                    elif packet_command == 6: # ble_rsp_system_get_connections
                        maxconn = struct.unpack('<B', self.bgapi_rx_payload[:1])[0]
                        self.ble_rsp_system_get_connections({ 'maxconn': maxconn })
                    elif packet_command == 7: # ble_rsp_system_read_memory
                        address, data_len = struct.unpack('<IB', self.bgapi_rx_payload[:5])
                        data_data = self.bgapi_rx_payload[5:]
                        self.ble_rsp_system_read_memory({ 'address': address, 'data': data_data })
                    elif packet_command == 8: # ble_rsp_system_get_info
                        major, minor, patch, build, ll_version, protocol_version, hw = struct.unpack('<HHHHHBB', self.bgapi_rx_payload[:12])
                        self.ble_rsp_system_get_info({ 'major': major, 'minor': minor, 'patch': patch, 'build': build, 'll_version': ll_version, 'protocol_version': protocol_version, 'hw': hw })
                    elif packet_command == 9: # ble_rsp_system_endpoint_tx
                        result = struct.unpack('<H', self.bgapi_rx_payload[:2])[0]
                        self.ble_rsp_system_endpoint_tx({ 'result': result })
                    elif packet_command == 10: # ble_rsp_system_whitelist_append
                        result = struct.unpack('<H', self.bgapi_rx_payload[:2])[0]
                        self.ble_rsp_system_whitelist_append({ 'result': result })
                    elif packet_command == 11: # ble_rsp_system_whitelist_remove
                        result = struct.unpack('<H', self.bgapi_rx_payload[:2])[0]
                        self.ble_rsp_system_whitelist_remove({ 'result': result })
                    elif packet_command == 12: # ble_rsp_system_whitelist_clear
                        self.ble_rsp_system_whitelist_clear({  })
                    elif packet_command == 13: # ble_rsp_system_endpoint_rx
                        result, data_len = struct.unpack('<HB', self.bgapi_rx_payload[:3])
                        data_data = self.bgapi_rx_payload[3:]
                        self.ble_rsp_system_endpoint_rx({ 'result': result, 'data': data_data })
                    elif packet_command == 14: # ble_rsp_system_endpoint_set_watermarks
                        result = struct.unpack('<H', self.bgapi_rx_payload[:2])[0]
                        self.ble_rsp_system_endpoint_set_watermarks({ 'result': result })
                elif packet_class == 1:
                    if packet_command == 0: # ble_rsp_flash_ps_defrag
                        self.ble_rsp_flash_ps_defrag({  })
                    elif packet_command == 1: # ble_rsp_flash_ps_dump
                        self.ble_rsp_flash_ps_dump({  })
                    elif packet_command == 2: # ble_rsp_flash_ps_erase_all
                        self.ble_rsp_flash_ps_erase_all({  })
                    elif packet_command == 3: # ble_rsp_flash_ps_save
                        result = struct.unpack('<H', self.bgapi_rx_payload[:2])[0]
                        self.ble_rsp_flash_ps_save({ 'result': result })
                    elif packet_command == 4: # ble_rsp_flash_ps_load
                        result, value_len = struct.unpack('<HB', self.bgapi_rx_payload[:3])
                        value_data = self.bgapi_rx_payload[3:]
                        self.ble_rsp_flash_ps_load({ 'result': result, 'value': value_data })
                    elif packet_command == 5: # ble_rsp_flash_ps_erase
                        self.ble_rsp_flash_ps_erase({  })
                    elif packet_command == 6: # ble_rsp_flash_erase_page
                        result = struct.unpack('<H', self.bgapi_rx_payload[:2])[0]
                        self.ble_rsp_flash_erase_page({ 'result': result })
                    elif packet_command == 7: # ble_rsp_flash_write_words
                        self.ble_rsp_flash_write_words({  })
                elif packet_class == 2:
                    if packet_command == 0: # ble_rsp_attributes_write
                        result = struct.unpack('<H', self.bgapi_rx_payload[:2])[0]
                        self.ble_rsp_attributes_write({ 'result': result })
                    elif packet_command == 1: # ble_rsp_attributes_read
                        handle, offset, result, value_len = struct.unpack('<HHHB', self.bgapi_rx_payload[:7])
                        value_data = self.bgapi_rx_payload[7:]
                        self.ble_rsp_attributes_read({ 'handle': handle, 'offset': offset, 'result': result, 'value': value_data })
                    elif packet_command == 2: # ble_rsp_attributes_read_type
                        handle, result, value_len = struct.unpack('<HHB', self.bgapi_rx_payload[:5])
                        value_data = self.bgapi_rx_payload[5:]
                        self.ble_rsp_attributes_read_type({ 'handle': handle, 'result': result, 'value': value_data })
                    elif packet_command == 3: # ble_rsp_attributes_user_read_response
                        self.ble_rsp_attributes_user_read_response({  })
                    elif packet_command == 4: # ble_rsp_attributes_user_write_response
                        self.ble_rsp_attributes_user_write_response({  })
                elif packet_class == 3:
                    if packet_command == 0: # ble_rsp_connection_disconnect
                        connection, result = struct.unpack('<BH', self.bgapi_rx_payload[:3])
                        self.ble_rsp_connection_disconnect({ 'connection': connection, 'result': result })
                    elif packet_command == 1: # ble_rsp_connection_get_rssi
                        connection, rssi = struct.unpack('<Bb', self.bgapi_rx_payload[:2])
                        self.ble_rsp_connection_get_rssi({ 'connection': connection, 'rssi': rssi })
                    elif packet_command == 2: # ble_rsp_connection_update
                        connection, result = struct.unpack('<BH', self.bgapi_rx_payload[:3])
                        self.ble_rsp_connection_update({ 'connection': connection, 'result': result })
                    elif packet_command == 3: # ble_rsp_connection_version_update
                        connection, result = struct.unpack('<BH', self.bgapi_rx_payload[:3])
                        self.ble_rsp_connection_version_update({ 'connection': connection, 'result': result })
                    elif packet_command == 4: # ble_rsp_connection_channel_map_get
                        connection, map_len = struct.unpack('<BB', self.bgapi_rx_payload[:2])
                        map_data = self.bgapi_rx_payload[2:]
                        self.ble_rsp_connection_channel_map_get({ 'connection': connection, 'map': map_data })
                    elif packet_command == 5: # ble_rsp_connection_channel_map_set
                        connection, result = struct.unpack('<BH', self.bgapi_rx_payload[:3])
                        self.ble_rsp_connection_channel_map_set({ 'connection': connection, 'result': result })
                    elif packet_command == 6: # ble_rsp_connection_features_get
                        connection, result = struct.unpack('<BH', self.bgapi_rx_payload[:3])
                        self.ble_rsp_connection_features_get({ 'connection': connection, 'result': result })
                    elif packet_command == 7: # ble_rsp_connection_get_status
                        connection = struct.unpack('<B', self.bgapi_rx_payload[:1])[0]
                        self.ble_rsp_connection_get_status({ 'connection': connection })
                    elif packet_command == 8: # ble_rsp_connection_raw_tx
                        connection = struct.unpack('<B', self.bgapi_rx_payload[:1])[0]
                        self.ble_rsp_connection_raw_tx({ 'connection': connection })
                elif packet_class == 4:
                    if packet_command == 0: # ble_rsp_attclient_find_by_type_value
                        connection, result = struct.unpack('<BH', self.bgapi_rx_payload[:3])
                        self.ble_rsp_attclient_find_by_type_value({ 'connection': connection, 'result': result })
                    elif packet_command == 1: # ble_rsp_attclient_read_by_group_type
                        connection, result = struct.unpack('<BH', self.bgapi_rx_payload[:3])
                        self.ble_rsp_attclient_read_by_group_type({ 'connection': connection, 'result': result })
                    elif packet_command == 2: # ble_rsp_attclient_read_by_type
                        connection, result = struct.unpack('<BH', self.bgapi_rx_payload[:3])
                        self.ble_rsp_attclient_read_by_type({ 'connection': connection, 'result': result })
                    elif packet_command == 3: # ble_rsp_attclient_find_information
                        connection, result = struct.unpack('<BH', self.bgapi_rx_payload[:3])
                        self.ble_rsp_attclient_find_information({ 'connection': connection, 'result': result })
                    elif packet_command == 4: # ble_rsp_attclient_read_by_handle
                        connection, result = struct.unpack('<BH', self.bgapi_rx_payload[:3])
                        self.ble_rsp_attclient_read_by_handle({ 'connection': connection, 'result': result })
                    elif packet_command == 5: # ble_rsp_attclient_attribute_write
                        connection, result = struct.unpack('<BH', self.bgapi_rx_payload[:3])
                        self.ble_rsp_attclient_attribute_write({ 'connection': connection, 'result': result })
                    elif packet_command == 6: # ble_rsp_attclient_write_command
                        connection, result = struct.unpack('<BH', self.bgapi_rx_payload[:3])
                        self.ble_rsp_attclient_write_command({ 'connection': connection, 'result': result })
                    elif packet_command == 7: # ble_rsp_attclient_indicate_confirm
                        result = struct.unpack('<H', self.bgapi_rx_payload[:2])[0]
                        self.ble_rsp_attclient_indicate_confirm({ 'result': result })
                    elif packet_command == 8: # ble_rsp_attclient_read_long
                        connection, result = struct.unpack('<BH', self.bgapi_rx_payload[:3])
                        self.ble_rsp_attclient_read_long({ 'connection': connection, 'result': result })
                    elif packet_command == 9: # ble_rsp_attclient_prepare_write
                        connection, result = struct.unpack('<BH', self.bgapi_rx_payload[:3])
                        self.ble_rsp_attclient_prepare_write({ 'connection': connection, 'result': result })
                    elif packet_command == 10: # ble_rsp_attclient_execute_write
                        connection, result = struct.unpack('<BH', self.bgapi_rx_payload[:3])
                        self.ble_rsp_attclient_execute_write({ 'connection': connection, 'result': result })
                    elif packet_command == 11: # ble_rsp_attclient_read_multiple
                        connection, result = struct.unpack('<BH', self.bgapi_rx_payload[:3])
                        self.ble_rsp_attclient_read_multiple({ 'connection': connection, 'result': result })
                elif packet_class == 5:
                    if packet_command == 0: # ble_rsp_sm_encrypt_start
                        handle, result = struct.unpack('<BH', self.bgapi_rx_payload[:3])
                        self.ble_rsp_sm_encrypt_start({ 'handle': handle, 'result': result })
                    elif packet_command == 1: # ble_rsp_sm_set_bondable_mode
                        self.ble_rsp_sm_set_bondable_mode({  })
                    elif packet_command == 2: # ble_rsp_sm_delete_bonding
                        result = struct.unpack('<H', self.bgapi_rx_payload[:2])[0]
                        self.ble_rsp_sm_delete_bonding({ 'result': result })
                    elif packet_command == 3: # ble_rsp_sm_set_parameters
                        self.ble_rsp_sm_set_parameters({  })
                    elif packet_command == 4: # ble_rsp_sm_passkey_entry
                        result = struct.unpack('<H', self.bgapi_rx_payload[:2])[0]
                        self.ble_rsp_sm_passkey_entry({ 'result': result })
                    elif packet_command == 5: # ble_rsp_sm_get_bonds
                        bonds = struct.unpack('<B', self.bgapi_rx_payload[:1])[0]
                        self.ble_rsp_sm_get_bonds({ 'bonds': bonds })
                    elif packet_command == 6: # ble_rsp_sm_set_oob_data
                        self.ble_rsp_sm_set_oob_data({  })
                elif packet_class == 6:
                    if packet_command == 0: # ble_rsp_gap_set_privacy_flags
                        self.ble_rsp_gap_set_privacy_flags({  })
                    elif packet_command == 1: # ble_rsp_gap_set_mode
                        result = struct.unpack('<H', self.bgapi_rx_payload[:2])[0]
                        self.ble_rsp_gap_set_mode({ 'result': result })
                    elif packet_command == 2: # ble_rsp_gap_discover
                        result = struct.unpack('<H', self.bgapi_rx_payload[:2])[0]
                        self.ble_rsp_gap_discover({ 'result': result })
                    elif packet_command == 3: # ble_rsp_gap_connect_direct
                        result, connection_handle = struct.unpack('<HB', self.bgapi_rx_payload[:3])
                        self.ble_rsp_gap_connect_direct({ 'result': result, 'connection_handle': connection_handle })
                    elif packet_command == 4: # ble_rsp_gap_end_procedure
                        result = struct.unpack('<H', self.bgapi_rx_payload[:2])[0]
                        self.ble_rsp_gap_end_procedure({ 'result': result })
                    elif packet_command == 5: # ble_rsp_gap_connect_selective
                        result, connection_handle = struct.unpack('<HB', self.bgapi_rx_payload[:3])
                        self.ble_rsp_gap_connect_selective({ 'result': result, 'connection_handle': connection_handle })
                    elif packet_command == 6: # ble_rsp_gap_set_filtering
                        result = struct.unpack('<H', self.bgapi_rx_payload[:2])[0]
                        self.ble_rsp_gap_set_filtering({ 'result': result })
                    elif packet_command == 7: # ble_rsp_gap_set_scan_parameters
                        result = struct.unpack('<H', self.bgapi_rx_payload[:2])[0]
                        self.ble_rsp_gap_set_scan_parameters({ 'result': result })
                    elif packet_command == 8: # ble_rsp_gap_set_adv_parameters
                        result = struct.unpack('<H', self.bgapi_rx_payload[:2])[0]
                        self.ble_rsp_gap_set_adv_parameters({ 'result': result })
                    elif packet_command == 9: # ble_rsp_gap_set_adv_data
                        result = struct.unpack('<H', self.bgapi_rx_payload[:2])[0]
                        self.ble_rsp_gap_set_adv_data({ 'result': result })
                    elif packet_command == 10: # ble_rsp_gap_set_directed_connectable_mode
                        result = struct.unpack('<H', self.bgapi_rx_payload[:2])[0]
                        self.ble_rsp_gap_set_directed_connectable_mode({ 'result': result })
                elif packet_class == 7:
                    if packet_command == 0: # ble_rsp_hardware_io_port_config_irq
                        result = struct.unpack('<H', self.bgapi_rx_payload[:2])[0]
                        self.ble_rsp_hardware_io_port_config_irq({ 'result': result })
                    elif packet_command == 1: # ble_rsp_hardware_set_soft_timer
                        result = struct.unpack('<H', self.bgapi_rx_payload[:2])[0]
                        self.ble_rsp_hardware_set_soft_timer({ 'result': result })
                    elif packet_command == 2: # ble_rsp_hardware_adc_read
                        result = struct.unpack('<H', self.bgapi_rx_payload[:2])[0]
                        self.ble_rsp_hardware_adc_read({ 'result': result })
                    elif packet_command == 3: # ble_rsp_hardware_io_port_config_direction
                        result = struct.unpack('<H', self.bgapi_rx_payload[:2])[0]
                        self.ble_rsp_hardware_io_port_config_direction({ 'result': result })
                    elif packet_command == 4: # ble_rsp_hardware_io_port_config_function
                        result = struct.unpack('<H', self.bgapi_rx_payload[:2])[0]
                        self.ble_rsp_hardware_io_port_config_function({ 'result': result })
                    elif packet_command == 5: # ble_rsp_hardware_io_port_config_pull
                        result = struct.unpack('<H', self.bgapi_rx_payload[:2])[0]
                        self.ble_rsp_hardware_io_port_config_pull({ 'result': result })
                    elif packet_command == 6: # ble_rsp_hardware_io_port_write
                        result = struct.unpack('<H', self.bgapi_rx_payload[:2])[0]
                        self.ble_rsp_hardware_io_port_write({ 'result': result })
                    elif packet_command == 7: # ble_rsp_hardware_io_port_read
                        result, port, data = struct.unpack('<HBB', self.bgapi_rx_payload[:4])
                        self.ble_rsp_hardware_io_port_read({ 'result': result, 'port': port, 'data': data })
                    elif packet_command == 8: # ble_rsp_hardware_spi_config
                        result = struct.unpack('<H', self.bgapi_rx_payload[:2])[0]
                        self.ble_rsp_hardware_spi_config({ 'result': result })
                    elif packet_command == 9: # ble_rsp_hardware_spi_transfer
                        result, channel, data_len = struct.unpack('<HBB', self.bgapi_rx_payload[:4])
                        data_data = self.bgapi_rx_payload[4:]
                        self.ble_rsp_hardware_spi_transfer({ 'result': result, 'channel': channel, 'data': data_data })
                    elif packet_command == 10: # ble_rsp_hardware_i2c_read
                        result, data_len = struct.unpack('<HB', self.bgapi_rx_payload[:3])
                        data_data = self.bgapi_rx_payload[3:]
                        self.ble_rsp_hardware_i2c_read({ 'result': result, 'data': data_data })
                    elif packet_command == 11: # ble_rsp_hardware_i2c_write
                        written = struct.unpack('<B', self.bgapi_rx_payload[:1])[0]
                        self.ble_rsp_hardware_i2c_write({ 'written': written })
                    elif packet_command == 12: # ble_rsp_hardware_set_txpower
                        self.ble_rsp_hardware_set_txpower({  })
                    elif packet_command == 13: # ble_rsp_hardware_timer_comparator
                        result = struct.unpack('<H', self.bgapi_rx_payload[:2])[0]
                        self.ble_rsp_hardware_timer_comparator({ 'result': result })
                elif packet_class == 8:
                    if packet_command == 0: # ble_rsp_test_phy_tx
                        self.ble_rsp_test_phy_tx({  })
                    elif packet_command == 1: # ble_rsp_test_phy_rx
                        self.ble_rsp_test_phy_rx({  })
                    elif packet_command == 2: # ble_rsp_test_phy_end
                        counter = struct.unpack('<H', self.bgapi_rx_payload[:2])[0]
                        self.ble_rsp_test_phy_end({ 'counter': counter })
                    elif packet_command == 3: # ble_rsp_test_phy_reset
                        self.ble_rsp_test_phy_reset({  })
                    elif packet_command == 4: # ble_rsp_test_get_channel_map
                        channel_map_len = struct.unpack('<B', self.bgapi_rx_payload[:1])[0]
                        channel_map_data = self.bgapi_rx_payload[1:]
                        self.ble_rsp_test_get_channel_map({ 'channel_map': channel_map_data })
                    elif packet_command == 5: # ble_rsp_test_debug
                        output_len = struct.unpack('<B', self.bgapi_rx_payload[:1])[0]
                        output_data = self.bgapi_rx_payload[1:]
                        self.ble_rsp_test_debug({ 'output': output_data })
                self.busy = False
                self.on_idle()
            elif packet_type & 0x88 == 0x80:
                # 0x80 = BLE event packet
                if packet_class == 0:
                    if packet_command == 0: # ble_evt_system_boot
                        major, minor, patch, build, ll_version, protocol_version, hw = struct.unpack('<HHHHHBB', self.bgapi_rx_payload[:12])
                        self.ble_evt_system_boot({ 'major': major, 'minor': minor, 'patch': patch, 'build': build, 'll_version': ll_version, 'protocol_version': protocol_version, 'hw': hw })
                        self.busy = False
                        self.on_idle()
                    elif packet_command == 1: # ble_evt_system_debug
                        data_len = struct.unpack('<B', self.bgapi_rx_payload[:1])[0]
                        data_data = self.bgapi_rx_payload[1:]
                        self.ble_evt_system_debug({ 'data': data_data })
                    elif packet_command == 2: # ble_evt_system_endpoint_watermark_rx
                        endpoint, data = struct.unpack('<BB', self.bgapi_rx_payload[:2])
                        self.ble_evt_system_endpoint_watermark_rx({ 'endpoint': endpoint, 'data': data })
                    elif packet_command == 3: # ble_evt_system_endpoint_watermark_tx
                        endpoint, data = struct.unpack('<BB', self.bgapi_rx_payload[:2])
                        self.ble_evt_system_endpoint_watermark_tx({ 'endpoint': endpoint, 'data': data })
                    elif packet_command == 4: # ble_evt_system_script_failure
                        address, reason = struct.unpack('<HH', self.bgapi_rx_payload[:4])
                        self.ble_evt_system_script_failure({ 'address': address, 'reason': reason })
                    elif packet_command == 5: # ble_evt_system_no_license_key
                        self.ble_evt_system_no_license_key({  })
                elif packet_class == 1:
                    if packet_command == 0: # ble_evt_flash_ps_key
                        key, value_len = struct.unpack('<HB', self.bgapi_rx_payload[:3])
                        value_data = self.bgapi_rx_payload[3:]
                        self.ble_evt_flash_ps_key({ 'key': key, 'value': value_data })
                elif packet_class == 2:
                    if packet_command == 0: # ble_evt_attributes_value
                        connection, reason, handle, offset, value_len = struct.unpack('<BBHHB', self.bgapi_rx_payload[:7])
                        value_data = self.bgapi_rx_payload[7:]
                        self.ble_evt_attributes_value({ 'connection': connection, 'reason': reason, 'handle': handle, 'offset': offset, 'value': value_data })
                    elif packet_command == 1: # ble_evt_attributes_user_read_request
                        connection, handle, offset, maxsize = struct.unpack('<BHHB', self.bgapi_rx_payload[:6])
                        self.ble_evt_attributes_user_read_request({ 'connection': connection, 'handle': handle, 'offset': offset, 'maxsize': maxsize })
                    elif packet_command == 2: # ble_evt_attributes_status
                        handle, flags = struct.unpack('<HB', self.bgapi_rx_payload[:3])
                        self.ble_evt_attributes_status({ 'handle': handle, 'flags': flags })
                elif packet_class == 3:
                    if packet_command == 0: # ble_evt_connection_status
                        connection, flags, address, address_type, conn_interval, timeout, latency, bonding = struct.unpack('<BB6sBHHHB', self.bgapi_rx_payload[:16])
                        address = address
                        self.ble_evt_connection_status({ 'connection': connection, 'flags': flags, 'address': address, 'address_type': address_type, 'conn_interval': conn_interval, 'timeout': timeout, 'latency': latency, 'bonding': bonding })
                    elif packet_command == 1: # ble_evt_connection_version_ind
                        connection, vers_nr, comp_id, sub_vers_nr = struct.unpack('<BBHH', self.bgapi_rx_payload[:6])
                        self.ble_evt_connection_version_ind({ 'connection': connection, 'vers_nr': vers_nr, 'comp_id': comp_id, 'sub_vers_nr': sub_vers_nr })
                    elif packet_command == 2: # ble_evt_connection_feature_ind
                        connection, features_len = struct.unpack('<BB', self.bgapi_rx_payload[:2])
                        features_data = self.bgapi_rx_payload[2:]
                        self.ble_evt_connection_feature_ind({ 'connection': connection, 'features': features_data })
                    elif packet_command == 3: # ble_evt_connection_raw_rx
                        connection, data_len = struct.unpack('<BB', self.bgapi_rx_payload[:2])
                        data_data = self.bgapi_rx_payload[2:]
                        self.ble_evt_connection_raw_rx({ 'connection': connection, 'data': data_data })
                    elif packet_command == 4: # ble_evt_connection_disconnected
                        connection, reason = struct.unpack('<BH', self.bgapi_rx_payload[:3])
                        self.ble_evt_connection_disconnected({ 'connection': connection, 'reason': reason })
                elif packet_class == 4:
                    if packet_command == 0: # ble_evt_attclient_indicated
                        connection, attrhandle = struct.unpack('<BH', self.bgapi_rx_payload[:3])
                        self.ble_evt_attclient_indicated({ 'connection': connection, 'attrhandle': attrhandle })
                    elif packet_command == 1: # ble_evt_attclient_procedure_completed
                        connection, result, chrhandle = struct.unpack('<BHH', self.bgapi_rx_payload[:5])
                        self.ble_evt_attclient_procedure_completed({ 'connection': connection, 'result': result, 'chrhandle': chrhandle })
                    elif packet_command == 2: # ble_evt_attclient_group_found
                        connection, start, end, uuid_len = struct.unpack('<BHHB', self.bgapi_rx_payload[:6])
                        uuid_data = self.bgapi_rx_payload[6:]
                        self.ble_evt_attclient_group_found({ 'connection': connection, 'start': start, 'end': end, 'uuid': uuid_data })
                    elif packet_command == 3: # ble_evt_attclient_attribute_found
                        connection, chrdecl, value, properties, uuid_len = struct.unpack('<BHHBB', self.bgapi_rx_payload[:7])
                        uuid_data = self.bgapi_rx_payload[7:]
                        self.ble_evt_attclient_attribute_found({ 'connection': connection, 'chrdecl': chrdecl, 'value': value, 'properties': properties, 'uuid': uuid_data })
                    elif packet_command == 4: # ble_evt_attclient_find_information_found
                        connection, chrhandle, uuid_len = struct.unpack('<BHB', self.bgapi_rx_payload[:4])
                        uuid_data = self.bgapi_rx_payload[4:]
                        self.ble_evt_attclient_find_information_found({ 'connection': connection, 'chrhandle': chrhandle, 'uuid': uuid_data })
                    elif packet_command == 5: # ble_evt_attclient_attribute_value
                        connection, atthandle, type, value_len = struct.unpack('<BHBB', self.bgapi_rx_payload[:5])
                        value_data = self.bgapi_rx_payload[5:]
                        self.ble_evt_attclient_attribute_value({ 'connection': connection, 'atthandle': atthandle, 'type': type, 'value': value_data })
                    elif packet_command == 6: # ble_evt_attclient_read_multiple_response
                        connection, handles_len = struct.unpack('<BB', self.bgapi_rx_payload[:2])
                        handles_data = self.bgapi_rx_payload[2:]
                        self.ble_evt_attclient_read_multiple_response({ 'connection': connection, 'handles': handles_data })
                elif packet_class == 5:
                    if packet_command == 0: # ble_evt_sm_smp_data
                        handle, packet, data_len = struct.unpack('<BBB', self.bgapi_rx_payload[:3])
                        data_data = self.bgapi_rx_payload[3:]
                        self.ble_evt_sm_smp_data({ 'handle': handle, 'packet': packet, 'data': data_data })
                    elif packet_command == 1: # ble_evt_sm_bonding_fail
                        handle, result = struct.unpack('<BH', self.bgapi_rx_payload[:3])
                        self.ble_evt_sm_bonding_fail({ 'handle': handle, 'result': result })
                    elif packet_command == 2: # ble_evt_sm_passkey_display
                        handle, passkey = struct.unpack('<BI', self.bgapi_rx_payload[:5])
                        self.ble_evt_sm_passkey_display({ 'handle': handle, 'passkey': passkey })
                    elif packet_command == 3: # ble_evt_sm_passkey_request
                        handle = struct.unpack('<B', self.bgapi_rx_payload[:1])[0]
                        self.ble_evt_sm_passkey_request({ 'handle': handle })
                    elif packet_command == 4: # ble_evt_sm_bond_status
                        bond, keysize, mitm, keys = struct.unpack('<BBBB', self.bgapi_rx_payload[:4])
                        self.ble_evt_sm_bond_status({ 'bond': bond, 'keysize': keysize, 'mitm': mitm, 'keys': keys })
                elif packet_class == 6:
                    if packet_command == 0: # ble_evt_gap_scan_response
                        rssi, packet_type, sender, address_type, bond, data_len = struct.unpack('<bB6sBBB', self.bgapi_rx_payload[:11])
                        sender = sender
                        data_data = self.bgapi_rx_payload[11:]
                        self.ble_evt_gap_scan_response({ 'rssi': rssi, 'packet_type': packet_type, 'sender': sender, 'address_type': address_type, 'bond': bond, 'data': data_data })
                    elif packet_command == 1: # ble_evt_gap_mode_changed
                        discover, connect = struct.unpack('<BB', self.bgapi_rx_payload[:2])
                        self.ble_evt_gap_mode_changed({ 'discover': discover, 'connect': connect })
                elif packet_class == 7:
                    if packet_command == 0: # ble_evt_hardware_io_port_status
                        timestamp, port, irq, state = struct.unpack('<IBBB', self.bgapi_rx_payload[:7])
                        self.ble_evt_hardware_io_port_status({ 'timestamp': timestamp, 'port': port, 'irq': irq, 'state': state })
                    elif packet_command == 1: # ble_evt_hardware_soft_timer
                        handle = struct.unpack('<B', self.bgapi_rx_payload[:1])[0]
                        self.ble_evt_hardware_soft_timer({ 'handle': handle })
                    elif packet_command == 2: # ble_evt_hardware_adc_result
                        input, value = struct.unpack('<Bh', self.bgapi_rx_payload[:3])
                        self.ble_evt_hardware_adc_result({ 'input': input, 'value': value })
            elif packet_type & 0x88 == 0x08:
                # 0x08 = wifi response packet
                if packet_class == 0:
                    if packet_command == 0: # wifi_rsp_dfu_reset
                        self.wifi_rsp_dfu_reset({  })
                        self.busy = False
                        self.on_idle()
                    elif packet_command == 1: # wifi_rsp_dfu_flash_set_address
                        result = struct.unpack('<H', self.bgapi_rx_payload[:2])[0]
                        self.wifi_rsp_dfu_flash_set_address({ 'result': result })
                    elif packet_command == 2: # wifi_rsp_dfu_flash_upload
                        result = struct.unpack('<H', self.bgapi_rx_payload[:2])[0]
                        self.wifi_rsp_dfu_flash_upload({ 'result': result })
                    elif packet_command == 3: # wifi_rsp_dfu_flash_upload_finish
                        result = struct.unpack('<H', self.bgapi_rx_payload[:2])[0]
                        self.wifi_rsp_dfu_flash_upload_finish({ 'result': result })
                elif packet_class == 1:
                    if packet_command == 0: # wifi_rsp_system_sync
                        self.wifi_rsp_system_sync({  })
                    elif packet_command == 1: # wifi_rsp_system_reset
                        self.wifi_rsp_system_reset({  })
                    elif packet_command == 2: # wifi_rsp_system_hello
                        self.wifi_rsp_system_hello({  })
                    elif packet_command == 3: # wifi_rsp_system_set_max_power_saving_state
                        result = struct.unpack('<H', self.bgapi_rx_payload[:2])[0]
                        self.wifi_rsp_system_set_max_power_saving_state({ 'result': result })
                elif packet_class == 2:
                    if packet_command == 0: # wifi_rsp_config_get_mac
                        result, hw_interface = struct.unpack('<HB', self.bgapi_rx_payload[:3])
                        self.wifi_rsp_config_get_mac({ 'result': result, 'hw_interface': hw_interface })
                    elif packet_command == 1: # wifi_rsp_config_set_mac
                        result, hw_interface = struct.unpack('<HB', self.bgapi_rx_payload[:3])
                        self.wifi_rsp_config_set_mac({ 'result': result, 'hw_interface': hw_interface })
                elif packet_class == 3:
                    if packet_command == 0: # wifi_rsp_sme_wifi_on
                        result = struct.unpack('<H', self.bgapi_rx_payload[:2])[0]
                        self.wifi_rsp_sme_wifi_on({ 'result': result })
                    elif packet_command == 1: # wifi_rsp_sme_wifi_off
                        result = struct.unpack('<H', self.bgapi_rx_payload[:2])[0]
                        self.wifi_rsp_sme_wifi_off({ 'result': result })
                    elif packet_command == 2: # wifi_rsp_sme_power_on
                        result = struct.unpack('<H', self.bgapi_rx_payload[:2])[0]
                        self.wifi_rsp_sme_power_on({ 'result': result })
                    elif packet_command == 3: # wifi_rsp_sme_start_scan
                        result = struct.unpack('<H', self.bgapi_rx_payload[:2])[0]
                        self.wifi_rsp_sme_start_scan({ 'result': result })
                    elif packet_command == 4: # wifi_rsp_sme_stop_scan
                        result = struct.unpack('<H', self.bgapi_rx_payload[:2])[0]
                        self.wifi_rsp_sme_stop_scan({ 'result': result })
                    elif packet_command == 5: # wifi_rsp_sme_set_password
                        status = struct.unpack('<B', self.bgapi_rx_payload[:1])[0]
                        self.wifi_rsp_sme_set_password({ 'status': status })
                    elif packet_command == 6: # wifi_rsp_sme_connect_bssid
                        result, hw_interface = struct.unpack('<HB', self.bgapi_rx_payload[:3])
                        self.wifi_rsp_sme_connect_bssid({ 'result': result, 'hw_interface': hw_interface })
                    elif packet_command == 7: # wifi_rsp_sme_connect_ssid
                        result, hw_interface = struct.unpack('<HB', self.bgapi_rx_payload[:3])
                        self.wifi_rsp_sme_connect_ssid({ 'result': result, 'hw_interface': hw_interface })
                    elif packet_command == 8: # wifi_rsp_sme_disconnect
                        result, hw_interface = struct.unpack('<HB', self.bgapi_rx_payload[:3])
                        self.wifi_rsp_sme_disconnect({ 'result': result, 'hw_interface': hw_interface })
                    elif packet_command == 9: # wifi_rsp_sme_set_scan_channels
                        result = struct.unpack('<H', self.bgapi_rx_payload[:2])[0]
                        self.wifi_rsp_sme_set_scan_channels({ 'result': result })
                elif packet_class == 4:
                    if packet_command == 0: # wifi_rsp_tcpip_start_tcp_server
                        result, endpoint = struct.unpack('<HB', self.bgapi_rx_payload[:3])
                        self.wifi_rsp_tcpip_start_tcp_server({ 'result': result, 'endpoint': endpoint })
                    elif packet_command == 1: # wifi_rsp_tcpip_tcp_connect
                        result, endpoint = struct.unpack('<HB', self.bgapi_rx_payload[:3])
                        self.wifi_rsp_tcpip_tcp_connect({ 'result': result, 'endpoint': endpoint })
                    elif packet_command == 2: # wifi_rsp_tcpip_start_udp_server
                        result, endpoint = struct.unpack('<HB', self.bgapi_rx_payload[:3])
                        self.wifi_rsp_tcpip_start_udp_server({ 'result': result, 'endpoint': endpoint })
                    elif packet_command == 3: # wifi_rsp_tcpip_udp_connect
                        result, endpoint = struct.unpack('<HB', self.bgapi_rx_payload[:3])
                        self.wifi_rsp_tcpip_udp_connect({ 'result': result, 'endpoint': endpoint })
                    elif packet_command == 4: # wifi_rsp_tcpip_configure
                        result = struct.unpack('<H', self.bgapi_rx_payload[:2])[0]
                        self.wifi_rsp_tcpip_configure({ 'result': result })
                    elif packet_command == 5: # wifi_rsp_tcpip_dns_configure
                        result = struct.unpack('<H', self.bgapi_rx_payload[:2])[0]
                        self.wifi_rsp_tcpip_dns_configure({ 'result': result })
                    elif packet_command == 6: # wifi_rsp_tcpip_dns_gethostbyname
                        result = struct.unpack('<H', self.bgapi_rx_payload[:2])[0]
                        self.wifi_rsp_tcpip_dns_gethostbyname({ 'result': result })
                elif packet_class == 5:
                    if packet_command == 0: # wifi_rsp_endpoint_send
                        result, endpoint = struct.unpack('<HB', self.bgapi_rx_payload[:3])
                        self.wifi_rsp_endpoint_send({ 'result': result, 'endpoint': endpoint })
                    elif packet_command == 1: # wifi_rsp_endpoint_set_streaming
                        result, endpoint = struct.unpack('<HB', self.bgapi_rx_payload[:3])
                        self.wifi_rsp_endpoint_set_streaming({ 'result': result, 'endpoint': endpoint })
                    elif packet_command == 2: # wifi_rsp_endpoint_set_active
                        result, endpoint = struct.unpack('<HB', self.bgapi_rx_payload[:3])
                        self.wifi_rsp_endpoint_set_active({ 'result': result, 'endpoint': endpoint })
                    elif packet_command == 3: # wifi_rsp_endpoint_set_streaming_destination
                        result, endpoint = struct.unpack('<HB', self.bgapi_rx_payload[:3])
                        self.wifi_rsp_endpoint_set_streaming_destination({ 'result': result, 'endpoint': endpoint })
                    elif packet_command == 4: # wifi_rsp_endpoint_close
                        result, endpoint = struct.unpack('<HB', self.bgapi_rx_payload[:3])
                        self.wifi_rsp_endpoint_close({ 'result': result, 'endpoint': endpoint })
                elif packet_class == 6:
                    if packet_command == 0: # wifi_rsp_hardware_set_soft_timer
                        result = struct.unpack('<H', self.bgapi_rx_payload[:2])[0]
                        self.wifi_rsp_hardware_set_soft_timer({ 'result': result })
                    elif packet_command == 1: # wifi_rsp_hardware_external_interrupt_config
                        result = struct.unpack('<H', self.bgapi_rx_payload[:2])[0]
                        self.wifi_rsp_hardware_external_interrupt_config({ 'result': result })
                    elif packet_command == 2: # wifi_rsp_hardware_change_notification_config
                        result = struct.unpack('<H', self.bgapi_rx_payload[:2])[0]
                        self.wifi_rsp_hardware_change_notification_config({ 'result': result })
                    elif packet_command == 3: # wifi_rsp_hardware_change_notification_pullup
                        result = struct.unpack('<H', self.bgapi_rx_payload[:2])[0]
                        self.wifi_rsp_hardware_change_notification_pullup({ 'result': result })
                    elif packet_command == 4: # wifi_rsp_hardware_io_port_config_direction
                        result = struct.unpack('<H', self.bgapi_rx_payload[:2])[0]
                        self.wifi_rsp_hardware_io_port_config_direction({ 'result': result })
                    elif packet_command == 5: # wifi_rsp_hardware_io_port_config_open_drain
                        result = struct.unpack('<H', self.bgapi_rx_payload[:2])[0]
                        self.wifi_rsp_hardware_io_port_config_open_drain({ 'result': result })
                    elif packet_command == 6: # wifi_rsp_hardware_io_port_write
                        result = struct.unpack('<H', self.bgapi_rx_payload[:2])[0]
                        self.wifi_rsp_hardware_io_port_write({ 'result': result })
                    elif packet_command == 7: # wifi_rsp_hardware_io_port_read
                        result, port, data = struct.unpack('<HBH', self.bgapi_rx_payload[:5])
                        self.wifi_rsp_hardware_io_port_read({ 'result': result, 'port': port, 'data': data })
                    elif packet_command == 8: # wifi_rsp_hardware_output_compare
                        result = struct.unpack('<H', self.bgapi_rx_payload[:2])[0]
                        self.wifi_rsp_hardware_output_compare({ 'result': result })
                    elif packet_command == 9: # wifi_rsp_hardware_adc_read
                        result, input, value = struct.unpack('<HBH', self.bgapi_rx_payload[:5])
                        self.wifi_rsp_hardware_adc_read({ 'result': result, 'input': input, 'value': value })
                elif packet_class == 7:
                    if packet_command == 0: # wifi_rsp_flash_ps_defrag
                        result = struct.unpack('<H', self.bgapi_rx_payload[:2])[0]
                        self.wifi_rsp_flash_ps_defrag({ 'result': result })
                    elif packet_command == 1: # wifi_rsp_flash_ps_dump
                        result = struct.unpack('<H', self.bgapi_rx_payload[:2])[0]
                        self.wifi_rsp_flash_ps_dump({ 'result': result })
                    elif packet_command == 2: # wifi_rsp_flash_ps_erase_all
                        result = struct.unpack('<H', self.bgapi_rx_payload[:2])[0]
                        self.wifi_rsp_flash_ps_erase_all({ 'result': result })
                    elif packet_command == 3: # wifi_rsp_flash_ps_save
                        result = struct.unpack('<H', self.bgapi_rx_payload[:2])[0]
                        self.wifi_rsp_flash_ps_save({ 'result': result })
                    elif packet_command == 4: # wifi_rsp_flash_ps_load
                        result, value_len = struct.unpack('<HB', self.bgapi_rx_payload[:3])
                        value_data = self.bgapi_rx_payload[3:]
                        self.wifi_rsp_flash_ps_load({ 'result': result, 'value': value_data })
                    elif packet_command == 5: # wifi_rsp_flash_ps_erase
                        result = struct.unpack('<H', self.bgapi_rx_payload[:2])[0]
                        self.wifi_rsp_flash_ps_erase({ 'result': result })
                elif packet_class == 8:
                    if packet_command == 0: # wifi_rsp_i2c_start_read
                        result = struct.unpack('<H', self.bgapi_rx_payload[:2])[0]
                        self.wifi_rsp_i2c_start_read({ 'result': result })
                    elif packet_command == 1: # wifi_rsp_i2c_start_write
                        result = struct.unpack('<H', self.bgapi_rx_payload[:2])[0]
                        self.wifi_rsp_i2c_start_write({ 'result': result })
                    elif packet_command == 2: # wifi_rsp_i2c_stop
                        result = struct.unpack('<H', self.bgapi_rx_payload[:2])[0]
                        self.wifi_rsp_i2c_stop({ 'result': result })
                self.busy = False
                self.on_idle()
            else:
                # 0x88 = wifi event packet
                if packet_class == 0:
                    if packet_command == 0: # wifi_evt_dfu_boot
                        version = struct.unpack('<I', self.bgapi_rx_payload[:4])[0]
                        self.wifi_evt_dfu_boot({ 'version': version })
                        self.busy = False
                        self.on_idle()
                elif packet_class == 1:
                    if packet_command == 0: # wifi_evt_system_boot
                        major, minor, patch, build, bootloader_version, tcpip_version, hw = struct.unpack('<HHHHHHH', self.bgapi_rx_payload[:14])
                        self.wifi_evt_system_boot({ 'major': major, 'minor': minor, 'patch': patch, 'build': build, 'bootloader_version': bootloader_version, 'tcpip_version': tcpip_version, 'hw': hw })
                    elif packet_command == 1: # wifi_evt_system_state
                        state = struct.unpack('<H', self.bgapi_rx_payload[:2])[0]
                        self.wifi_evt_system_state({ 'state': state })
                    elif packet_command == 2: # wifi_evt_system_sw_exception
                        address, type = struct.unpack('<IB', self.bgapi_rx_payload[:5])
                        self.wifi_evt_system_sw_exception({ 'address': address, 'type': type })
                    elif packet_command == 3: # wifi_evt_system_power_saving_state
                        state = struct.unpack('<B', self.bgapi_rx_payload[:1])[0]
                        self.wifi_evt_system_power_saving_state({ 'state': state })
                elif packet_class == 2:
                    if packet_command == 0: # wifi_evt_config_mac_address
                        hw_interface = struct.unpack('<B', self.bgapi_rx_payload[:1])[0]
                        self.wifi_evt_config_mac_address({ 'hw_interface': hw_interface })
                elif packet_class == 3:
                    if packet_command == 0: # wifi_evt_sme_wifi_is_on
                        result = struct.unpack('<H', self.bgapi_rx_payload[:2])[0]
                        self.wifi_evt_sme_wifi_is_on({ 'result': result })
                    elif packet_command == 1: # wifi_evt_sme_wifi_is_off
                        result = struct.unpack('<H', self.bgapi_rx_payload[:2])[0]
                        self.wifi_evt_sme_wifi_is_off({ 'result': result })
                    elif packet_command == 2: # wifi_evt_sme_scan_result
                        channel, rssi, snr, secure, ssid_len = struct.unpack('<bhbBB', self.bgapi_rx_payload[:6])
                        ssid_data = self.bgapi_rx_payload[6:]
                        self.wifi_evt_sme_scan_result({ 'channel': channel, 'rssi': rssi, 'snr': snr, 'secure': secure, 'ssid': ssid_data })
                    elif packet_command == 3: # wifi_evt_sme_scan_result_drop
                        self.wifi_evt_sme_scan_result_drop({  })
                    elif packet_command == 4: # wifi_evt_sme_scanned
                        status = struct.unpack('<b', self.bgapi_rx_payload[:1])[0]
                        self.wifi_evt_sme_scanned({ 'status': status })
                    elif packet_command == 5: # wifi_evt_sme_connected
                        status, hw_interface = struct.unpack('<bB', self.bgapi_rx_payload[:2])
                        self.wifi_evt_sme_connected({ 'status': status, 'hw_interface': hw_interface })
                    elif packet_command == 6: # wifi_evt_sme_disconnected
                        reason, hw_interface = struct.unpack('<HB', self.bgapi_rx_payload[:3])
                        self.wifi_evt_sme_disconnected({ 'reason': reason, 'hw_interface': hw_interface })
                    elif packet_command == 7: # wifi_evt_sme_interface_status
                        hw_interface, status = struct.unpack('<BB', self.bgapi_rx_payload[:2])
                        self.wifi_evt_sme_interface_status({ 'hw_interface': hw_interface, 'status': status })
                    elif packet_command == 8: # wifi_evt_sme_connect_failed
                        reason, hw_interface = struct.unpack('<HB', self.bgapi_rx_payload[:3])
                        self.wifi_evt_sme_connect_failed({ 'reason': reason, 'hw_interface': hw_interface })
                    elif packet_command == 9: # wifi_evt_sme_connect_retry
                        hw_interface = struct.unpack('<B', self.bgapi_rx_payload[:1])[0]
                        self.wifi_evt_sme_connect_retry({ 'hw_interface': hw_interface })
                elif packet_class == 4:
                    if packet_command == 0: # wifi_evt_tcpip_configuration
                        use_dhcp = struct.unpack('<B', self.bgapi_rx_payload[:1])[0]
                        self.wifi_evt_tcpip_configuration({ 'use_dhcp': use_dhcp })
                    elif packet_command == 1: # wifi_evt_tcpip_dns_configuration
                        index = struct.unpack('<B', self.bgapi_rx_payload[:1])[0]
                        self.wifi_evt_tcpip_dns_configuration({ 'index': index })
                    elif packet_command == 2: # wifi_evt_tcpip_endpoint_status
                        endpoint, local_port, remote_port = struct.unpack('<BHH', self.bgapi_rx_payload[:5])
                        self.wifi_evt_tcpip_endpoint_status({ 'endpoint': endpoint, 'local_port': local_port, 'remote_port': remote_port })
                    elif packet_command == 3: # wifi_evt_tcpip_dns_gethostbyname_result
                        result, name_len = struct.unpack('<HB', self.bgapi_rx_payload[:3])
                        name_data = self.bgapi_rx_payload[3:]
                        self.wifi_evt_tcpip_dns_gethostbyname_result({ 'result': result, 'name': name_data })
                elif packet_class == 5:
                    if packet_command == 0: # wifi_evt_endpoint_syntax_error
                        endpoint = struct.unpack('<B', self.bgapi_rx_payload[:1])[0]
                        self.wifi_evt_endpoint_syntax_error({ 'endpoint': endpoint })
                    elif packet_command == 1: # wifi_evt_endpoint_data
                        endpoint, data_len = struct.unpack('<BB', self.bgapi_rx_payload[:2])
                        data_data = self.bgapi_rx_payload[2:]
                        self.wifi_evt_endpoint_data({ 'endpoint': endpoint, 'data': data_data })
                    elif packet_command == 2: # wifi_evt_endpoint_status
                        endpoint, type, streaming, destination, active = struct.unpack('<BIBbB', self.bgapi_rx_payload[:8])
                        self.wifi_evt_endpoint_status({ 'endpoint': endpoint, 'type': type, 'streaming': streaming, 'destination': destination, 'active': active })
                    elif packet_command == 3: # wifi_evt_endpoint_closing
                        reason, endpoint = struct.unpack('<HB', self.bgapi_rx_payload[:3])
                        self.wifi_evt_endpoint_closing({ 'reason': reason, 'endpoint': endpoint })
                elif packet_class == 6:
                    if packet_command == 0: # wifi_evt_hardware_soft_timer
                        handle = struct.unpack('<B', self.bgapi_rx_payload[:1])[0]
                        self.wifi_evt_hardware_soft_timer({ 'handle': handle })
                    elif packet_command == 1: # wifi_evt_hardware_change_notification
                        timestamp = struct.unpack('<I', self.bgapi_rx_payload[:4])[0]
                        self.wifi_evt_hardware_change_notification({ 'timestamp': timestamp })
                    elif packet_command == 2: # wifi_evt_hardware_external_interrupt
                        irq, timestamp = struct.unpack('<BI', self.bgapi_rx_payload[:5])
                        self.wifi_evt_hardware_external_interrupt({ 'irq': irq, 'timestamp': timestamp })
                elif packet_class == 7:
                    if packet_command == 0: # wifi_evt_flash_ps_key
                        key, value_len = struct.unpack('<HB', self.bgapi_rx_payload[:3])
                        value_data = self.bgapi_rx_payload[3:]
                        self.wifi_evt_flash_ps_key({ 'key': key, 'value': value_data })

# ================================================================




# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function

DTYPE_1ST = 0 << 6
DTYPE_2ND = 1 << 6
DTYPE_EXT = 2 << 6
MSK_DTYPE = 0xC0

EXT_TYPE_SYNCFINISH = 0x1f
EXT_TYPE_ACK = 0x00
EXT_TYPE_NACK = 0x01
MSK_EXT_DTYPE = 0x3f

class SensorManager:
    def __init__(self):
        self.readState = 0
        self.sensorValue = [-1] * 11;
        self.statWrite = 0    # 0: none 1: waiting -1: NACK
        self.statSyncServo = 0

    def parseData(self, data):
        #print('Num data: ', len(data))
        for i in range(0, len(data)):
            #print(data[i])
            if (data[i] & MSK_DTYPE) == DTYPE_1ST:
                if not self.readState == 0:
                    continue
                self.data1 = data[i]
                self.readState = 1
            elif (data[i] & MSK_DTYPE) == DTYPE_2ND:
                if not self.readState == 1:
                    continue
                self.data2 = data[i]
                port = (self.data1 >> 2) & 0x0f
                val = ((self.data1 & 0x03) << 6) | (self.data2 & 0x3f)
                self.sensorValue[port] = val
                self.readState = 0
                #print('sensor update')
            elif (data[i] & MSK_DTYPE) == DTYPE_EXT:
                if data[i] & MSK_EXT_DTYPE == EXT_TYPE_SYNCFINISH:
                    #print('sync stop')
                    self.statSyncServo = 0
                if data[i] & MSK_EXT_DTYPE == EXT_TYPE_ACK:
                    #print('ACK received.')
                    self.statWrite = 0
                if data[i] & MSK_EXT_DTYPE == EXT_TYPE_NACK:
                    self.statWrite = -1

    def getValue(self, index):
        return self.sensorValue[index]

    def startWaitingWriteResponse(self):
        self.statWrite = 1

    def getWriteFlag(self):
        return self.statWrite

    def startSyncServo(self):
        self.statSyncServo = 1

    def getSyncServoFlag(self):
        return self.statSyncServo



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function

import serial
import time
import datetime
import threading
import struct
import sys
import pkg_resources

debugOn = False

Resp_STK_OK     = 0x10
Resp_STK_INSYNC = 0x14
Resp_STK_NOSYNC = 0x15

ser = None
LOCK = threading.Lock()

def _open(comPort):
	try:
		global ser
		ser = serial.Serial(comPort, 115200, timeout=0.1)
		ser.write_timeout = 0.5
		while not ser.writable():
			time.sleep(0.01)
		debugPrint('ready')
		time.sleep(1)
		debugPrint('start')
	except serial.serialutil.SerialException:
		print('Could not find the port.')

def _close():
	global ser
	if not ser == None:
		debugPrint('Disconnected.')
		time.sleep(0.1)
		ser.close()

def _send(data):
	global LOCK
	with LOCK:
		try:
			global ser
			ser.write(bytes(i for i in data))
		except:
			debugPrint('write exception')

def _recv(buf, length):
	global LOCK
	with LOCK:
		retval = 0
		totalRetval = 0
		startTime = datetime.datetime.now()
		ser.timeout = 0.1
	
		while True:
			tmpbuf = ser.read(length)
			retval = len(tmpbuf)
			if(retval > 0):
				buf[totalRetval:totalRetval+retval] = tmpbuf
				totalRetval = totalRetval + retval
				startTime = datetime.datetime.now()
			if(totalRetval >= length):
				break
	
			endTime = datetime.datetime.now()
			if((endTime - startTime).microseconds > 250000):
				break
	
		return retval

def _drain():
	retval = 0
	startTime = datetime.datetime.now()
	while True:
		retval = ser.read(1)
		if not retval is None:
			debugPrint(retval)
		currentTime = datetime.datetime.now()
		if((currentTime - startTime).microseconds > 500000):
			debugPrint('drain timeout')
			break

def _getsync():
	buf = []
	buf.append(0x30)
	buf.append(0x20)

	_send(buf)
	_drain()

	_send(buf)
	_drain()

	_send(buf)
	ret = _recv(buf, 2)
	if ret <= 0:
		return -1
	else:
		return buf

def _setParameters():
	buf = []
	buf.append(0x42)
	buf.append(0x86)
	buf.append(0)
	buf.append(0)
	buf.append(1)
	buf.append(1)
	buf.append(1)
	buf.append(1)
	buf.append(3)
	buf.append(0xff)
	buf.append(0xff)
	buf.append(0xff)
	buf.append(0xff)
	buf.append(0)
	buf.append(0x80)
	buf.append(2)
	buf.append(0)
	buf.append(0)
	buf.append(0)
	buf.append(0x40)
	buf.append(0)
	buf.append(0x20)

	_send(buf)
	ret = _recv(buf, 2)
	return ret

def _setExtendedParameters():
	buf = []
	buf.append(0x45)
	buf.append(0x05)
	buf.append(0x04)
	buf.append(0xd7)
	buf.append(0xc2)
	buf.append(0)
	buf.append(0x20)

	_send(buf)
	ret = _recv(buf, 2)
	return ret

def _enter_programmode():
	buf = [0x50, 0x20]
	_send(buf)
	ret = _recv(buf, 2)
	return ret

def _leave_programmode():
	buf = [0x51, 0x20]
	_send(buf)
	ret = _recv(buf, 2)
	return ret


def _loadaddr(addr):
	buf = []
	buf.append(0x55)    # Cmnd_STK_LOAD_ADDRESS
	buf.append( addr       & 0xff)
	buf.append((addr >> 8) & 0xff)
	buf.append(0x20)    # Sync_CRC_EOP
	#buf = (0x55, addr & 0xff, (addr >> 8) & 0xff, 0x20)
	#print('loadaddr')
	#print(buf)
	_send(buf)
	#ret = recv(buf, 2)
	rbuf = []
	ret = _recv(rbuf, 2)
	return ret

def _paged_write(data):
	buf = []
	send_size = len(data)

	buf.append(0x64)      # Cmnd_STK_PROG_PAGE
	buf.append((send_size >> 8) & 0xff)
	buf.append((send_size ) & 0xff)
	buf.append(0x46)      # 'F' memory type
	buf.extend(data)
	buf.append(0x20)      # Sync_CRC_EOP
	
	# !! data send command !!
	#print(buf)
	_send(buf)
	ret = _recv(buf, 1)
	if(ret <= 0):
		return -1
	if buf[0] == Resp_STK_NOSYNC:
		return -3
	elif not buf[0] == Resp_STK_INSYNC:
		print('protocol error')
		return -4

	return 0

def _transferMain(data):
	page_size = 128
	n_bytes = len(data)

	addr = 0
	while addr < n_bytes:
		tries = 0
		bRetry = True
		buf = []
		while bRetry:
			bRetry = False
			tries = tries + 1

			_loadaddr(addr // 2)
	
			send_size = page_size
			if(addr + send_size > n_bytes):
				send_size -= (addr + send_size) - n_bytes
	
			ret = _paged_write(data[addr:addr+send_size])
			if(ret == -1):
				return -1
			elif ret == -3:
				if tries > 33:
					print('Can\'t get into sync')
					return -3
				if _getsync() < 0:
					return -1
				bRetry = True
				print('Retry')
				continue
			elif ret == -4:
				return -4

		ret = _recv(buf, 1)
		if ret <= 0:
			return -1
		if not buf[0] == Resp_STK_OK:
			print('protocol error')
			return -5
	
		addr = addr + page_size
		print("#", end="")
		sys.stdout.flush()

	print("")

	return 0

def _readHex(buf):
	resource_package = __name__  # Could be any module/package name
	resource_path = '/'.join(('data', 'firmware.hex'))  # Do not use os.path.join(), see below

	# or for a file-like stream:
	template = pkg_resources.resource_stream(resource_package, resource_path)
	for line in template.readlines():
		strLine = str(line, 'ascii')
		if strLine[7:9] == "01":
			debugPrint('skip')
		else:
			tmp = strLine[9:len(strLine)-3]
			for i in range(len(tmp) // 2):
				start = i*2
				stbyte = tmp[start:start+2]
				buf.append(int(stbyte, 16))

def execute(port, hexFile='firmware.hex'):
	tStart = datetime.datetime.now()
	_open(port)
	
	data = []
	_readHex(data)

	print('Start uploading a firmware.')
	for i in range(5):
		debugPrint(('Get sync:', i+1))
		ret = _getsync()
		debugPrint(ret)
		if (not ret == -1) and (ret[0] == 20) and (ret[1] == 16):
			break

	debugPrint('Set Parameters')
	ret = _setParameters()
	debugPrint(ret)
	if ret <= 0:
		_close()
		sys.exit()

	debugPrint('Set Extended Parameters')
	ret = _setExtendedParameters()
	debugPrint(ret)

	debugPrint('Enter programmode')
	ret = _enter_programmode()
	debugPrint(ret)

	debugPrint('Transfer start')
	ret = _transferMain(data)
	if ret < 0:
		print('Upload failed.')
		_close()
		sys.exit()

	print('Successfully upload {} bytes.'.format(len(data)))

	debugPrint('Leave programmode')
	ret = _leave_programmode()
	debugPrint(ret)

	_close()

	elapsedTime = (datetime.datetime.now() - tStart)
	print('elapsed time: {}.{}'.format(elapsedTime.seconds, elapsedTime.microseconds//1000))

def debugPrint(msg):
	if(debugOn):
		print(msg)

if __name__ == '__main__':
	args = sys.argv
	if len(args) != 2:
		print('python upload.py COM*')
		quit()
	#readHex()
	execute(args[1])



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function

from .connector import *

# Action (LED/BUZZER)
#: Constants for turning LED/Buzzer off.
OFF = 0
#: Constants for turning LED/Buzzer on.
ON = 1

# DC motion
#: Constants for rotating DC motor in forward.
FWD = 0
#: Constants for rotating DC motor in backward.
BCK = 1
#: Constants for stopping DC motor with brake.
BRAKE = 2
#: Constants for stopping DC motor without brake.
COAST = 3

# Sync servo action
START = 0
STOP  = 1

# Connector (DC)
#: DC motor connector M1.
M1 = ConnectorDC(0)
#: DC motor connector M2.
M2 = ConnectorDC(1)
# Connector (SERVO)
#: Servomotor connector D2.
D2 = ConnectorServo(0)
#: Servomotor connector D4.
D4 = ConnectorServo(1)
#: Servomotor connector D7.
D7 = ConnectorServo(2)
#: Servomotor connector D8.
D8 = ConnectorServo(3)
#: Servomotor connector D9.
D9 = ConnectorServo(4)
#: Servomotor connector D10.
D10 = ConnectorServo(5)
#: Servomotor connector D11.
D11 = ConnectorServo(6)
#: Servomotor connector D12.
D12 = ConnectorServo(7)
# Connector (SENSOR)
#: Analog sensor connector A0.
A0 = ConnectorSensor(0)
#: Analog sensor connector A1.
A1 = ConnectorSensor(1)
#: Analog sensor connector A2.
A2 = ConnectorSensor(2)
#: Analog sensor connector A3.
A3 = ConnectorSensor(3)
#: Analog sensor connector A4.
A4 = ConnectorSensor(4)
#: Analog sensor connector A5.
A5 = ConnectorSensor(5)
#: Analog sensor connector A6.
A6 = ConnectorSensor(6)
#: Analog sensor connector A7.
A7 = ConnectorSensor(7)

# Sound ID
#: Sound constant for "Do".
DO  = 60
#: Sound constant for "Do#".
DO_ = 61
#: Sound constant for "Re".
RE  = 62
#: Sound constant for "Re#".
RE_ = 63
#: Sound constant for "Mi".
MI  = 64
#: Sound constant for "Fa".
FA  = 65
#: Sound constant for "Fa#".
FA_ = 66
#: Sound constant for "So".
SO  = 67
#: Sound constant for "So#".
SO_ = 68
#: Sound constant for "La".
LA  = 69
#: Sound constant for "La#".
LA_ = 70
#: Sound constant for "Ti".
TI  = 71




# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function

import numpy as np
import matplotlib.pyplot as plt
import multiprocessing
import threading
import time
from .const import *
from .part import *

from . import command

job = None
th_data = None
stop_event = None
stop_flag = None

class _PlotArea:
	"""
	クラスの説明
	"""
	def __init__(self, ax, x, num):
		self.ax = ax
		self.x = x
		self.y = []
		self.lines = []
		self.labels = []

		self.ax.legend(loc='upper left')

		for i in range(num):
			tmp_y = np.zeros_like(x)
			self.y.append(tmp_y)
			tmp_l, = ax.plot(x, tmp_y)
			self.lines.append(tmp_l)

	def setLabel(self, labels):
		"""
		関数の説明

		:type self: 型
		:type labels: 型
		:param self: 型
		:param labels: 型
		"""
		self.labels = labels

	def update(self, index, val):
		"""
		関数の説明

		:type self: 型
		:type index: 型
		:type val: 型
		:param self: 型
		:param index: 型
		:param val: 型
		"""
		add = np.array([val])
		self.y[index] = np.r_[self.y[index][1:], add]
		self.lines[index].set_label('%s: %d' % (self.labels[index], val))
		self.lines[index].set_data(self.x, self.y[index])

		self.ax.legend(loc='upper left')
		self.ax.set_xlim((self.x.min(), self.x.max()))

class _IOStat:
	"""
	クラスの説明
	"""
	def __init__(self):
		self.inited = {}

	def set(self, port, part):
		"""
		関数の説明

		:type port: 型
		:type part: 型
		:param port: 説明
		:param part: 説明
		"""
		if not port in self.inited:
			self.inited[port] = part

	def __filterDigital(self, x):
		"""
		関数の説明

		:type x: 型
		:param x: 説明
		:rtype: 戻り値の型
		:return: 戻り値の説明
		"""
		return isinstance(x[1], DigitalSensor)

	def __filterAnalog(self, x):
		"""
		関数の説明

		:type x: 型
		:param x: 説明
		:rtype: 戻り値の型
		:return: 戻り値の説明
		"""
		return isinstance(x[1], AnalogSensor)

	def __filterAccel(self, x):
		"""
		関数の説明

		:type x: 型
		:param x: 説明
		:rtype: 戻り値の型
		:return: 戻り値の説明
		"""
		return isinstance(x[1], Accelerometer)

	def getDigital(self):
		"""
		関数の説明

		:rtype: 戻り値の型
		:return: 戻り値の説明
		"""
		return list(filter(self.__filterDigital, self.inited.items()))
	
	def getAnalog(self):
		"""
		関数の説明

		:rtype: 戻り値の型
		:return: 戻り値の説明
		"""
		return list(filter(self.__filterAnalog, self.inited.items()))
	
	def getAccel(self):
		"""
		関数の説明

		:rtype: 戻り値の型
		:return: 戻り値の説明
		"""
		return list(filter(self.__filterAccel, self.inited.items()))

	def getNumOfTypes(self):
		"""
		関数の説明

		:rtype: 戻り値の型
		:return: 戻り値の説明
		"""
		num = 0
		if len(self.getDigital()) > 0:
			num += 1
		if len(self.getAnalog()) > 0:
			num += 1
		if len(self.getAccel()) > 0:
			num += 1
		return num

class _Graph:
	"""
	Graph for displaying sensor data.
	"""
	size_x = 20

	def __init__(self, io, q):
		self.io = io
		self.q = q

	def update(self, stop_flag):
		io = self.io
		numGraph = io.getNumOfTypes()
		print("Num of graph: %d" % numGraph)
		
		fig = plt.figure(figsize=(6 * numGraph, 5))
		x = np.arange(0, self.size_x, 1)
		pos = 0
		graphD = None
		graphA = None
		graphACC = None
	
		if not len(io.getDigital()) == 0:
			graphD = _PlotArea(fig.add_subplot(1, numGraph, pos + 1), x, len(io.getDigital()))
			for elm in io.getDigital():
				lbl = 'A' + str(elm[0].id) + ' ' + str(elm[1].name)
				graphD.labels.append(lbl)
			graphD.ax.set_ylim(-0.5, 1.5)
			pos = pos + 1

		if not len(io.getAnalog()) == 0:
			graphA = _PlotArea(fig.add_subplot(1, numGraph, pos + 1), x, len(io.getAnalog()))
			for elm in io.getAnalog():
				lbl = 'A' + str(elm[0].id) + ' ' + str(elm[1].name)
				graphA.labels.append(lbl)
			graphA.ax.set_ylim(0, 100)
			pos = pos + 1

		if not len(io.getAccel()) == 0:
			graphACC = _PlotArea(fig.add_subplot(1, numGraph, pos + 1), x, 3)
			graphACC.setLabel(['Acc X', 'Acc Y', 'Acc Z'])
			graphACC.ax.set_ylim(0, 100)
			pos = pos + 1
		
		while not stop_flag.is_set():
			try:
				x += 1

				if not graphD == None:
					sensorList = io.getDigital()
					for i, (port, elm) in enumerate(sensorList):
						#val = command.getSensor(port + 10)
						self.q.send([elm,])
						if self.q.poll(0.1):
							add = self.q.recv()
							val = add[0]
							graphD.update(i, val)
				if not graphA == None:
					sensorList = io.getAnalog()
					for i, (port, elm) in enumerate(sensorList):
						#val = command.getSensor(port + 10)
						self.q.send([elm,])
						if self.q.poll(0.1):
							add = self.q.recv()
							val = add[0]
							graphA.update(i, val)
				if not graphACC == None:
					sensorList = io.getAccel()
					for i, (port, elm) in enumerate(sensorList):
						self.q.send([elm,])
						if self.q.poll(0.1):
							add = self.q.recv()
							val = add[0]
							#val = (50, 50, 50)
							for i in range(len(val)):
								graphACC.update(i, val[i])

				plt.pause(0.01)
			#except KeyboardInterrupt:
			#	print("Keyboard interrupt@Graph update()")
			#	plt.close()
			#	break
			except:
				print("Exception in Graph update.")
				plt.close()
				#global stop_event
				#stop_event.set()
				break
			

def __sensorUpdate(q):
	"""
	関数の説明

	:type q: 型
	:param q: 説明
	"""
	global stop_event
	while not stop_event.is_set():
		#print("sensor update")
		try:
			if q.poll():
				#print("queue pool")
				rcv = q.recv()
				val = rcv[0].getValue()
				#print("queue send")
				q.send([val,])
		except KeyboardInterrupt:
			print("Keyboard interrupt@sensor update()")
			break
		except:
			print("Exception@sensorUpdate")
			break
	q.close()

def showGraph(sensors, wait=False):
	"""
	Open the graph window.

	:type sensors: Part[]
	:type wait: bool
	:param sensors: Array of Part objects to be displayed.
        :param wait: default: False
	"""
	io = _IOStat()

	for elm in sensors:
		io.set(elm.connector, elm)

	# Process for updating graph.
	global job, stop_flag
	data_conn, graph_conn = multiprocessing.Pipe()
	stop_flag = multiprocessing.Event()
	gr = _Graph(io, graph_conn)
	job = multiprocessing.Process(target=gr.update, args=(stop_flag,))
	job.start()

	# Thread for getting sensor data.
	global th_data, stop_event
	stop_event = threading.Event()
	th_data = threading.Thread(target=__sensorUpdate, args=(data_conn,))
	th_data.setDaemon(True);
	th_data.start()

	if wait:
		try:
			while True:
				pass
		except KeyboardInterrupt:
			print("Keyboard Interrupt@showGraph")
			hideGraph()
			command.stop()

def hideGraph():
	"""
	Close the graph window.
	"""
	global job, th_data, stop_event
	stop_event.set()
	#print('th_data join')
	th_data.join()
	stop_flag.set()
	#print('job join')
	job.join()




# -*- coding: utf-8 -*-
from __future__ import division, unicode_literals, print_function
"""
Studuino for Python
"""

import serial
import time
import datetime
import threading
import struct
from . import ble_wrapper
from studuino.sensor import SensorManager

ser = None
sensor = None
fConnected = False
servo_angles = [90] * 8
LOCK = threading.Lock()
LOCK_READ = threading.Lock()
fBLE = False

def start(comPort, baud=115200):
    """
    Connect to the Studuino board and run python script.

    :type comPort: str 
    :type baud: int 
    :param comPort: Serial port name
    :param baud: baud rate (default:115200)
    """
    global ser, sensor, fConnected
    try:
        ser = serial.Serial(comPort, baud, timeout=0)
        ser.write_timeout = 0.3
        #ser.read_timeout = 0.05 
        fConnected = True

        sensor = SensorManager()
        th_read = threading.Thread(target=_sensorRead)
        th_read.setDaemon(True)
        th_read.start()


        while not ser.writable():
            time.sleep(0.01)
        print('ready')
        #ser.read(2)
        print('start')
        time.sleep(2)
    except:
        print('Could not find the port.')

def startBLE(comPort, addr):
    """
    Connect to the Studuino board and run python script.

    :type comPort: str 
    :type addr: str 
    :param comPort: Serial port name
    :param addr: BLE module's MAC address (00:01:02:ab:cd:ef)
    """
    global fBLE, sensor
    fBLE = True

    sensor = SensorManager()
    ble_wrapper.start(comPort, addr)
    ble_wrapper.addReceiveHandler(_handle_ble_receive)
    ble_wrapper.start_sensor_read()
    time.sleep(1)  # The sleep is necessary for ignoring the communication between BLE module and Studuino at establishing the connection.

def stop():
    """
    Disonnect the Studuino board.
    """
    if fBLE:
        stopBLE()
    else:
        global ser, LOCK_READ, fConnected
        if not ser == None:
            fConnected = False
            print('Disconnected.')
            time.sleep(0.1)
            with LOCK_READ:
                ser.close()

def stopBLE():
    ble_wrapper.stop()

def _sensorRead():
    global sensor, fConnected, LOCK_READ
    while fConnected:
        try:
            with LOCK_READ:
                rcv = ser.read(5)
            sensor.parseData(rcv)
        except Exception as ex:
            print('Read error: ', ex)

def _handle_ble_receive(sender, earg):
    global sensor
    rcvData = earg['value']
    sensor.parseData(rcvData)

def __send(data1, data2):
    """
    Send data to the Studuino.

    :type data1: 型
    :type data2: 型
    :param data1: 説明
    :param data2: 説明
    """
    global ser, sensor, LOCK, fBLE
    data3 = (data1 + data2) & 0xff;
    msg = struct.pack(b'BBB', data1, data2, data3)

    if fBLE:
    # BLE
        # Without waiting
        # ble_wrapper.write_command(msg)

        # With waiting
        #sensor.startWaitingWriteResponse()
        #start = datetime.datetime.now()
        #ble_wrapper.write_command(msg)
        #print('before wait: ', start)
        #while (sensor.getWriteFlag() == 1 and (datetime.datetime.now() - start).seconds < 0.1):
            #pass
        #print('after wait: ', datetime.datetime.now())

        # With LOCK
        with LOCK:
            sensor.startWaitingWriteResponse()
            start = datetime.datetime.now()
            ble_wrapper.write_command(msg)
            while (sensor.getWriteFlag() == 1 and (datetime.datetime.now() - start).seconds < 1):
                #print('     waiting...')
                pass
    else:
    # USB
        try:
            with LOCK:
                sensor.startWaitingWriteResponse()
                start = datetime.datetime.now()
                ser.write(msg)
                while (sensor.getWriteFlag() == 1 and (datetime.datetime.now() - start).seconds < 1):
                    pass
        except:
            print('write exception')

def _init(pin, part):
    """
    Initialize the Studuino board.

    :type pin: 型
    :type part: 型
    :param pin: 説明
    :param part: 説明
    """
    data1 = 0xc0 + ((pin >> 1) & 0x0f)
    data2 = ((pin & 0x01) << 6) + part.id
    #print('Initialize', hex(data1), hex(data2))
    __send(data1, data2)

def _led(pin, action):
    """
    Control the LEDs.

    :type pin: 型
    :type action: 型
    :param pin: 説明
    :param action: 説明
    """
    id = pin - 10   # PinID to Axx ID (Axx begins with 10.)
    data1 = 0xb0 + ((id << 1) & 0x0e) + (action & 0x01)
    data2 = 0
    #print('LED', hex(data1), hex(data2))
    __send(data1, data2)

def _buzzer(pin, action, sound=0, duration=0):
    """
    Control the Buzzers.

    :type pin: 型
    :type action: 型
    :type sound: 型
    :type duration: 型
    :param pin: 説明
    :param action: 説明
    :param sound: 説明
    :param duration: 説明
    """
    id = pin - 10   # PinID to Ax ID (Ax begins with 10.)
    data1 = 0xa0 + ((id << 1) & 0x0e) + (action & 0x01)
    data2 = sound & 0x7f 
    #print('Buzzer', hex(data1), hex(data2))
    __send(data1, data2)
    if duration != 0:
        time.sleep(duration)
        __send(data1 & 0xfe, 0)

def _dcPower(pin, power):
    """
    Set power for the DC Motor.

    :type pin: 型
    :type power: 型
    :param pin: 説明
    :param power: 説明
    """
    data1 = 0x84 + ((pin << 3) & 0x0f)
    data2 = power & 0x7f
    #print('DC POWER', hex(data1), hex(data2))
    __send(data1, data2)

def _dc(pin, motion):
    """
    Move the DC Motor.

    :type pin: 型
    :type motion: 型
    :param pin: 説明
    :param motion: 説明
    """
    data1 = 0x80 + ((pin << 3) & 0x0f) + (1 + (motion >> 1 & 0x01))
    data2 = motion & 0x01
    #print('DC Motion', hex(data1), hex(data2))
    __send(data1, data2)

def _servo(pin, angle):
    """
    Move the Servo Motor.

    :type pin: 型
    :type angle: 型
    :param pin: 説明
    :param angle: 説明
    """
    global servo_angles
    id = pin - 2   # PinID to Dx ID (Dxx begins with 2.)
    servo_angles[id] = angle
    data1 = 0x90 + ((id << 1) & 0x0e) + (angle >> 7 & 0x01)
    data2 = angle & 0x7f 
    #print('Servomotor', hex(data1), hex(data2))
    __send(data1, data2)

# Sync servo action
START = 0
STOP  = 1

def _syncServo(action, delay=0):
    """
    Move the Servo Motor synchronously.

    :type action: 型
    :type delay: 型
    :param action: 説明
    :param delay: 説明
    """
    global fBLE
    data1 = 0xd0
    data2 = ((action & 0x01) << 6) + delay
    #print('Sync servo', hex(data1), hex(data2))
    __send(data1, data2)
    if action == START:
        sensor.startSyncServo()
    if action == STOP:
        if fBLE:
            ble_wrapper.waitSyncFinish()
        while sensor.getSyncServoFlag() == 1:
            pass

def _multiServo(pins, angles, delay=0):
    """
    Move prural Servo Motors synchronously.

    :type pins: 型
    :type angles: 型
    :type delay: 型
    :param pins: 説明
    :param angles: 説明
    :param delay: 説明
    """
    if not len(pins) == len(angles):
        return

    global servo_angles

    _syncServo(START, delay)
    deltaMax = 0
    for e1, e2 in zip(pins, angles):
        delta = abs(e2 - servo_angles[e1.id -2])
        if delta > deltaMax:
            deltaMax = delta
        _servo(e1.id, e2)
    #print('deltaMax:', deltaMax)
    _syncServo(STOP)
    time.sleep(delay * deltaMax / 1000)

def _getAngles():
    """
    説明を書いてください

    rtype: 戻り値の型
    return: 戻り値の説明
    """
    global servo_angles
    return servo_angles

"""
def _getSensor():
    global ser
    val = None
    rcv = ser.read()
    if not len(rcv) == 0:
        val = ord(rcv)
    return val
    """

def _initSensor(pin, snstype):
    data1 = 0xd1
    data2 = snstype << 4 | pin
    __send(data1, data2)

def _getSensorValue(pin):
    global sensor
    return sensor.getValue(pin)

def _getSensorValue_(pin):
    """
    Get the sensor value.

    :type pin: 型
    :param pin: 説明
    :rtype: 戻り値の型
    :return: 戻り値の説明
    """
    global ser, fBLE
    id = pin - 10   # PinID to Axx ID (Axx begins with 10.)
    data1 = 0xd1
    data2 = id
    #print('get sensor', hex(data1), hex(data2))
    val = None
    if fBLE:
    # BLE
        val = ble_wrapper.get_sensor((data1, data2))
        return val[0]
    else:
    # USB
        __send(data1, data2)
        rcv = ser.read()
        if not len(rcv) == 0:
            val = ord(rcv)
        return val

def _getAccel():
    """
    Get the accelerometer value.

    :rtype: 戻り値の型
    :return: 戻り値の説明
    """
    global ser, fBLE
    id = 14 - 10   # PinID(14) to Axx ID (Axx begins with 10.)
    data1 = 0xd1
    data2 = id
    #print('get sensor', hex(data1), hex(data2))
    val = None
    if fBLE:
    # BLE
        rcv = ble_wrapper.get_accel((data1, data2))
        val = rcv
    else:
    # USB
        __send(data1, data2)
        rcv = ser.read(3)
        if len(rcv) == 3:
            #print(rcv)
            val = struct.unpack(b'BBB', rcv)
    return val




# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function

from six import with_metaclass
from abc import ABCMeta, abstractmethod, abstractproperty
from . import command
from .const import *
from .connector import *


#class Part(metaclass=ABCMeta):
class Part(with_metaclass(ABCMeta, object)):
	"""
	Parts Class
	"""
	@abstractproperty
	def id(self):
		"""
		Part ID

		"""
		return

	@abstractproperty
	def name(self):
		"""
		Part name

		"""
		return

	@abstractmethod
	def _canAttach(self, connector):
		"""
		関数の説明

		:type connector: 型
		:param connector: 説明
		"""
		return

	def attach(self, connector):
		"""
		Attach the part to the specified connector.

		:type connector: Connector
		:param connector: A Connector object defined in studuino.connector.
		"""
		if self._canAttach(connector):
			self.connector = connector
			#command._init(connector._getGlobalId(), self)
			print("Part @%s is attached to connector %d." % (self.name, connector._getGlobalId()))
		else:
			raise InitException(self)
			print("Part @%s can't be attached to connector %d." % (self.name, connector._getGlobalId()))

class DCMotor(Part):
	"""
	DC Motor Class
	"""
	@property
	def id(self):
		"""
		DC motor's part ID

		:rtype: int
		:return: DC motor's part ID.
		"""
		return 0x01

	@property
	def name(self):
		"""
		関数の説明

		:rtype: str
		:return: "DC Motor"
		"""
		return "DC Motor"

	def _canAttach(self, connector):
		"""
		関数の説明

		:type connector: 型
		:param connector: 説明
		:rtype: bool
		:return: Whether this part is connectable to the specified connector.
		"""
		#return (connector >= 0 and connector <= 1)
		return isinstance(connector, ConnectorDC) 

	def setPower(self, power):
		"""
		Set the DC motor's power.

		:type power: int
		:param power: DC motor power [0-100].
		"""
		self.power = power
		command._dcPower(self.connector.id, power)

	def move(self, motion):
		"""
		Rotate the dc motor by specified motion type.

		:type motion: int
		:param motion: The number representing the motion type. [FWD|BCK]
		"""
		command._dc(self.connector.id, motion)

	def stop(self, motion):
		"""
		Stop the dc motor by specified motion type.

		:type motion: int
		:param motion: The number representing the motion type. [BRAKE|COAST]
		"""
		command._dc(self.connector.id, motion)

class Servomotor(Part):
	"""
	Servo Motor Class
	"""
	@property
	def id(self):
		"""
		Servomotor's part ID

		:rtype: int
		:return: 0x02
		"""
		return 0x02

	@property
	def name(self):
		"""
		Servomotor's part name

		:rtype: str
		:return: "Servomotor"
		"""
		return "Servomotor"

	def _canAttach(self, connector):
		"""
		関数の説明

		:type connector: 型
		:param connector: 説明
		:rtype: bool
		:return: 戻り値の説明
		"""
		return isinstance(connector, ConnectorServo) 

	def setAngle(self, angle):
		"""
		Set the servomotor's angle

		:type angle: int
		:param angle: Servomotor's angle in degree.
		"""
		self.angle = angle
		command._servo(self.connector._getGlobalId(), angle)

	@staticmethod
	def syncMove(servos, angles, delay):
		"""
		関数の説明

		:type servos: int[]
		:type angles: int[]
		:type delay: int
		:param servos: Array of servomotor port.
		:param angles: Array of servomotor angle.
		:param delay: Delay time per 1 degree [milliseconds].
		"""
		#print('syncmove')
		prev_angles = command._getAngles()
		command._syncServo(0, delay)
		deltaMax = 0
		for e1, e2 in zip(servos, angles):
			delta = abs(e2 - prev_angles[e1.id])
			if delta > deltaMax:
				deltaMax = delta
			e1.setAngle(e2)
		command._syncServo(1)

class LED(Part):
	"""
	LED Class
	"""
	@property
	def id(self):
		"""
		LED's part ID

		:rtype: int
		:return: 0x03
		"""
		return 0x03

	@property
	def name(self):
		"""
		LED's part name

		:rtype: str
		:return: "LED"
		"""
		return "LED"

	def _canAttach(self, connector):
		"""
		関数の説明

		:type connector: 型
		:param connector: 説明
		:rtype: 戻り値の型
		:return: 戻り値の説明
		"""
		return (isinstance(connector, ConnectorSensor) and connector.id <= 5)

	def on(self):
		"""
		Turning the LED on.

		"""
		command._led(self.connector._getGlobalId(), ON)

	def off(self):
		"""
		Turning the LED off.

		"""
		command._led(self.connector._getGlobalId(), OFF)

class Buzzer(Part):
	"""
	Buzzer Class
	"""
	@property
	def id(self):
		"""
		Buzzer's part ID

		:rtype: int
		:return: 0x04
		"""
		return 0x04

	@property
	def name(self):
		"""
		Buzzer's part name

		:rtype: str
		:return: "Buzzer"
		"""
		return "Buzzer"

	def _canAttach(self, connector):
		"""
		関数の説明

		:type connector: 型
		:param connector: 説明
		:rtype: 戻り値の型
		:return: 戻り値の説明
		"""
		return (isinstance(connector, ConnectorSensor) and connector.id <= 5)

	def on(self, sound, octave=0, duration=0):
		"""
		Play the specified type of sound.

		:type sound: int
		:type octave: int
		:type duration: int
		:param sound: Sound ID registered int studuino.const.
		:param octave: Octaves of the sound [0-8].
		:param duration: Duration of the sound in milliseconds.
		"""
		command._buzzer(self.connector._getGlobalId(), ON, sound + octave*12, duration)

	def off(self):
		"""
		Stop the buzzer.

		"""
		command._buzzer(self.connector._getGlobalId(), OFF)

class Sensor(Part):
	"""
	Sensor Class
	"""
	def getValue(self):
		"""
		Returning the sensor's value.

		:rtype: int
                :return: Sensor value (Digital Sensor:[0|1] Analog Sensor:[0-100])
		"""
		return command._getSensorValue(self.connector.id)

class DigitalSensor(Sensor):
	"""
	Digital Sensor Class
	"""
	def _canAttach(self, connector):
		"""
		関数の説明

		:type connector: 型
		:param connector: 説明
		:rtype: 戻り値の型
		:return: 戻り値の説明
		"""
		return (isinstance(connector, ConnectorSensor) and connector.id <= 5)

	def attach(self, connector):
		Part.attach(self, connector)
		command._initSensor(self.connector.id, 0)

class AnalogSensor(Sensor):
	"""
	Analog Sensor Class
	"""
	def _canAttach(self, connector):
		"""
		関数の説明

		:type connector: 型
		:param connector: 説明
		:rtype: 戻り値の型
		:return: 戻り値の説明
		"""
		return isinstance(connector, ConnectorSensor)

	def attach(self, connector):
		Part.attach(self, connector)
		command._initSensor(self.connector.id, 1)

class LightSensor(AnalogSensor):
	"""
	Light Sensor Class
	"""
	@property
	def id(self):
		"""
		Light sensor's part ID

		:rtype: int
		:return: 0x10
		"""
		return 0x10

	@property
	def name(self):
		"""
		Light sensor's part name

		:rtype: str
		:return: "Light sensor"
		"""
		return "Light sensor"

class TouchSensor(DigitalSensor):
	"""
	Touch Sensor Class
	"""
	@property
	def id(self):
		"""
		Touch sensor's part ID.

		:rtype: int
		:return: 0x11
		"""
		return 0x11

	@property
	def name(self):
		"""
		Touch sensor's part name.

		:rtype: str
		:return: "Touch sensor"
		"""
		return "Touch sensor"

class SoundSensor(AnalogSensor):
	"""
	Sound Sensor Class
	"""
	@property
	def id(self):
		"""
		Sound sensor's part ID.

		:rtype: int
		:return: 0x12
		"""
		return 0x12

	@property
	def name(self):
		"""
		Sound sensor's part name.

		:rtype: str
		:return: "Sound sensor"
		"""
		return "Sound sensor"

class IRPhotoreflector(AnalogSensor):
	"""
	IR photo reflector Class
	"""
	@property
	def id(self):
		"""
		IR Photoreflector's part ID.

		:rtype: int
		:return: 0x13
		"""
		return 0x13

	@property
	def name(self):
		"""
		IR Photoreflector's part name.

		:rtype: str
		:return: "IRPhotorefrector"
		"""
		return "IRPhotoreflector"

class Accelerometer(Sensor):
	"""
	Accelerometer Class
	"""
	@property
	def id(self):
		"""
		Acceoerometer's part ID

		:rtype: int
		:return: 0x14
		"""
		return 0x14
	@property
	def name(self):
		"""
		Acceoerometer's part name

		:rtype: str
		:return: "Accelerometer"
		"""
		return "Accelerometer"

	def _canAttach(self, connector):
		"""
		関数の説明

		:type connector: 型
		:param connector: 説明
		:rtype: 戻り値の型
		:return: 戻り値の説明
		"""
		return (isinstance(connector, ConnectorSensor), connector.id == 4)

	def attach(self, connector):
		Part.attach(self, connector)
		command._initSensor(self.connector.id, 2)

	def getValue(self):
		"""
		Returning the acceleration values.

		:rtype: int[]
		:return: Array of acceleration (x, y, z).
		"""
		return command._getAccel()

class PushSwitch(DigitalSensor):
	"""
	Push switch Class
	"""
	@property
	def id(self):
		"""
		Push switch's part ID

		:rtype: int
		:return: 0x15
		"""
		return 0x15

	@property
	def name(self):
		"""
		Push switch's part name

		:rtype: str
		:return: "Push switch"
		"""
		return "Push switch"

	def _canAttach(self, connector):
		"""
		関数の説明

		:type connector: 型
		:param connector: 説明
		:rtype: 戻り値の型
		:return: 戻り値の説明
		"""
		return (isinstance(connector, ConnectorSensor), connector.id <= 3)

class InitException(Exception):
	"""
	Init exeception Class
	"""
	def __init__(self, value):
		self.value = value


if __name__ == '__main__':
	from . import const
	dc = DCMotor()
	dc.attach(M1)
	sv = Servomotor()
	sv.attach(D2)

	ls = LightSensor()
	ls.attach(A0)
	ls.getValue()

	bt = PushSwitch()
	bt.attach(A1)
	bt.getValue()

	ts = TouchSensor()
	ts.attach(A2)
	ts.getValue()




# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function

from six import with_metaclass
from abc import ABCMeta, abstractmethod, abstractproperty

#class Connector(metaclass=ABCMeta):
class Connector(with_metaclass(ABCMeta, object)):
	"""
	Base class of connectors.
	"""
	@abstractproperty
	def _offset(self):
		return

	def __init__(self, id):
		self.id = id

	"""
	Returning the global ID of the connector.

	:rtype: int
	:rparam: Global ID of the connector.
	"""
	def _getGlobalId(self):
		return (self._offset + self.id)

class ConnectorDC(Connector):
	"""
	DC motor connector
	"""
	@property
	def _offset(self):
		return 0

class ConnectorServo(Connector):
	"""
	Servomotor connector
	"""
	@property
	def _offset(self):
		return 2

class ConnectorSensor(Connector):
	"""
	Sensor connector
	"""
	@property
	def _offset(self):
		return 10



if __name__ == '__main__':
	M1 = ConnectorDC(0)
	M2 = ConnectorDC(1)
	D2 = ConnectorServo(0)
	D4 = ConnectorServo(1)
	D7 = ConnectorServo(2)
	D8 = ConnectorServo(3)
	D9 = ConnectorServo(4)
	D10 = ConnectorServo(5)
	D11 = ConnectorServo(6)
	D12 = ConnectorServo(7)
	A0 = ConnectorSensor(0)
	A1 = ConnectorSensor(1)
	A2 = ConnectorSensor(2)
	A3 = ConnectorSensor(3)
	A4 = ConnectorSensor(4)
	A5 = ConnectorSensor(5)
	A6 = ConnectorSensor(6)
	A7 = ConnectorSensor(7)


	for elm in (M1, M2, D2, D4, D7, D8, D9, D10, D11, D12, A0, A1, A2, A3, A4, A5, A6, A7):
		print("Connector GID:%d ID:%d" % (elm._getGlobalId(), elm.id))




# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import unicode_literals
import os
from setuptools import setup
from setuptools import find_packages

try:
    with open('README.rst') as f:
        readme = f.read()
except IOError:
    readme = ''

def _requires_from_file(filename):
    return open(filename).read().splitlines()

# version
here = os.path.dirname(os.path.abspath(__file__))
version = next((line.split('=')[1].strip().replace("'", '')
                for line in open(os.path.join(here,
                                              'studuino',
                                              '__init__.py'))
                if line.startswith('__version__ = ')),
               '0.0.dev0')


def main():
    description = 'Python library for Studuino&ArtecRobo'

    setup(
        name='pystuduino',
        version='1.0.0',
        author='Artec Co., Ltd.',
        url='https://github.com/artec-kk/pystuduino',
        author_email='support@artec-kk.co.jp',
        maintainer='Artec Co., Ltd.',
        maintainer_email='support@artec-kk.co.jp',

        description=description,
        long_description=readme,
        packages=find_packages(),
        zip_safe=False,
        include_package_data=True,
        install_requires=['pyserial', 'numpy', 'matplotlib'],
        tests_require=[],
        setup_requires=[],
        entry_points="""
            # -*- Entry points: -*-
            [console_scripts]
            pkgdep = studuino.scripts.command:main
        """,
    )

if __name__ == '__main__':
    main()




# -*- coding: utf-8 -*-
from __future__ import division, unicode_literals, print_function

from .bglib import BGLib
import serial
import threading
import sys
import time
from datetime import datetime

fConnection = False
fProcedureStarted = False
fRecv = False
fAccel = False
fInSync = False
idxAccel = 0
rcvAccel = [0, 0, 0]

LOCK = threading.Lock()

def timeout_handler(sender, args):
    print( "BGAPI parser timed out.",)
    print( " Make sure the BLE device is in a known/idle state.")

def handle_connection_status(sender, earg):
    global fConnection
    val = earg['flags']
    # print( "Connection state flag: ", val)
    if val == 9:
        fConnection = True

def handle_procedure_completed(sender, earg):
    global fProcedureStarted
    result = earg['result']
    chrhandler = earg['chrhandle']
    # print( 'Procedure : ', result)
    if result == 0:
        fProcedureStarted = False

def handle_write_command(sender, earg):
    global fProcedureStarted
    result = earg['result']
    # print( 'Procedure : ', result)
    if result == 0:
        fProcedureStarted = False

def handle_receive(sender, earg):
    global rcvData, fRecv, fAccel, idxAccel, rcvAccel, fInSync
    rcvData = earg['value']
    # print('handle_receive', rcvData)
    if fAccel:
        rcvAccel[idxAccel] = rcvData[0]
        idxAccel = idxAccel + 1
        if idxAccel == 3:
            idxAccel = 0
            fAccel = False
    if rcvData[0] == 0x80:
        fRecv = True
    if rcvData[0] == 0x9f:
        fInSync = False

def write_command(data):
    global ser, ble, fRecv, LOCK
    ble.send_command(ser, ble.ble_cmd_attclient_write_command(0, 14, data))
    fRecv = False
    while (not fRecv):
        with LOCK:
            ble.check_activity(ser)

def get_sensor(data):
    global rcvData, fRecv
    fRecv = False
    write_command(data)
    # Wait until the connection is established.
    while (not fRecv):
        ble.check_activity(ser)
        time.sleep(0.01)
    return rcvData

def get_accel(data):
    global fAccel, rcvAccel, fRecv
    fAccel = True
    write_command(data)
    # Wait until the connection is established.
    while (fAccel):
        ble.check_activity(ser)
        time.sleep(0.01)
    return rcvAccel

def _sensorRead():
    global ble, ser, LOCK
    while True:
        with LOCK:
            ble.check_activity(ser)
        time.sleep(0.01)

def start_sensor_read():
    th_read = threading.Thread(target=_sensorRead)
    th_read.setDaemon(True)
    th_read.start()

def main():
    if len(sys.argv) != 2:
        print( "Usage: python %s port" % sys.argv[0])
        quit()

    port = sys.argv[1]
    start(port)

def start(port, address):
    global ble, ser, fConnection, fProcedureStarted
    ble = BGLib()
    ble.packet_mode = False
    ble.debug = False

    arr_addr = []
    arr_tmp = address.split(':')
    for i in range(len(arr_tmp) - 1, -1, -1):
        arr_addr.append(int(arr_tmp[i], 16))


    # add handler for BGAPI timeout condition (hopefully won't happen)
    ble.on_timeout += timeout_handler

    # Add Function
    ble.ble_evt_connection_status += handle_connection_status
    ble.ble_evt_attclient_procedure_completed += handle_procedure_completed
    ble.ble_rsp_attclient_write_command += handle_write_command
    ble.ble_evt_attclient_attribute_value += handle_receive

#    if len(sys.argv) != 2:
#        print( "Usage: python %s port" % sys.argv[0])
#        quit()

#    port = sys.argv[1]
    baudrate = 115200

    try:
        ser = serial.Serial(
            port=port, baudrate=baudrate, timeout=1, writeTimeout=1)
    except serial.SerialException as e:
        print( "\n" + ("=" * 20))
        print( "Port error (name='%s', baud='%ld'): %s" % (port, baudrate, e))
        print( "=" * 20)
        quit()

    # flush buffers
    ser.flushInput()
    ser.flushOutput()

    # disconnect if we are connected already
    ble.send_command(ser, ble.ble_cmd_connection_disconnect(0))
    ble.check_activity(ser, 1)

    # stop advertising if we are advertising already
    ble.send_command(ser, ble.ble_cmd_gap_set_mode(0, 0))
    ble.check_activity(ser, 1)

    # stop scanning if we are scanning already
    ble.send_command(ser, ble.ble_cmd_gap_end_procedure())
    ble.check_activity(ser, 1)

    # set advertising parameters
    ble.send_command(
        ser, ble.ble_cmd_gap_set_adv_parameters(0x640, 0x640, 0x07))
    ble.check_activity(ser, 1)

    # set TX power level
    ble.send_command(ser, ble.ble_cmd_hardware_set_txpower(15))
                     # range 0 to 15 (real TX power from -23 to +3dBm)
    ble.check_activity(ser, 1)

    ### connect to Studuino
    print("Connecting to the device.")
    #ble.send_command(ser, ble.ble_cmd_gap_connect_direct((0x77, 0x14, 0xe4, 0x2f, 0x1b, 0xf4), 1, 60, 76, 100, 0))
    ble.send_command(ser, ble.ble_cmd_gap_connect_direct(arr_addr, 1, 60, 76, 100, 0))
    # Wait until the connection is established.
    while (not fConnection):
        ble.check_activity(ser)
        time.sleep(0.01)


    ### Service discover
    # print("Service discover.")
    fProcedureStarted = True
    ble.send_command(ser, ble.ble_cmd_attclient_read_by_group_type(0, 1, 65535, (0x00, 0x28)))
    # Wait until the procedure is done.
    while (fProcedureStarted):
        ble.check_activity(ser)
        time.sleep(0.01)

    ### Characteristics discover
    # print("Charcteristics discover.")
    fProcedureStarted = True
    ble.send_command(ser, ble.ble_cmd_attclient_read_by_type(0, 9, 15, (0x03, 0x28)))
    # Wait until the procedure is done.
    while (fProcedureStarted):
        ble.check_activity(ser)
        time.sleep(0.01)

    ### Descriptor discover
    # print("Descriptor discover.")
    fProcedureStarted = True
    ble.send_command(ser, ble.ble_cmd_attclient_find_information(0, 12, 12))
    # Wait until the procedure is done.
    while (fProcedureStarted):
        ble.check_activity(ser)
        time.sleep(0.01)

    ### Read characteristics notification on
    # print("notification on.")
    fProcedureStarted = True
    ble.send_command(ser, ble.ble_cmd_attclient_read_long(0, 10))
    # Wait until the procedure is done.
    while (fProcedureStarted):
        ble.check_activity(ser)
        time.sleep(0.01)

    ### Set Enable notification 
    # print("set enable notification.")
    fProcedureStarted = True
    ble.send_command(ser, ble.ble_cmd_attclient_write_command(0, 12, (0x01, 0x00)))
    # Wait until the procedure is done.
    while (fProcedureStarted):
        ble.check_activity(ser)
        time.sleep(0.01)

    print("started.")

def stop():
    global ser, ble
    # disconnect if we are connected already
    ble.send_command(ser, ble.ble_cmd_connection_disconnect(0))
    ble.check_activity(ser, 1)

    # stop advertising if we are advertising already
    ble.send_command(ser, ble.ble_cmd_gap_set_mode(0, 0))
    ble.check_activity(ser, 1)

    # stop scanning if we are scanning already
    ble.send_command(ser, ble.ble_cmd_gap_end_procedure())
    ble.check_activity(ser, 1)

    print("stopped")

def addReceiveHandler(handler):
    ble.ble_evt_attclient_attribute_value += handler

def waitSyncFinish():
    global fInSync, LOCK
    fInSync = True
    while fInSync:
        with LOCK:
            ble.check_activity(ser)
            time.sleep(0.01)

if __name__ == "__main__":
    main()
